use strict;
use Test::Simple tests => 10;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath LoadClassByAlias ExistsObjectByPath );
use DE_EPAGES::Address::DAL::Address;

sub Main {
    ConnectTestSite();
    ok( Init(), "initialization" );

    my $Object = LoadObjectByPath('O1');
    ok( defined $Object, 'address exists');
    ok( $Object->get('Company') eq 'epages', 'company epages');
    ok( $Object->get('City') eq 'Jena', 'in jena');

    $Object->set({'Company'=>undef});
    ok( (not defined DAL_GetInfoAddressPerson($Object->id)) , 'person address deleted');

    $Object->set({'Company'=>'epages'});
    ok( $Object->get('Company') eq 'epages', 'company epages');

    $Object->set({'L1' => 'deutscher text'}, GetPKeyLanguageByCode('de'));
    $Object->set({'L1' => 'english text'}, GetPKeyLanguageByCode('en'));

    my $ObjectClone = $Object->clone({'Alias'=>'O2'});
    ok( defined $ObjectClone, 'clone exists');
    ok( $ObjectClone->get('Company') eq 'epages', 'clone company epages');
    ok( $ObjectClone->get('L1', GetPKeyLanguageByCode('en')) eq 'english text', 'clone localize');

    ok( Cleanup(), "cleanup" );
    DisconnectSite();
}

sub Cleanup {
    foreach my $Path (qw (O1 O2)) {
        if (ExistsObjectByPath($Path)) {
            my $object = LoadObjectByPath($Path);
            $object->delete;
        }
    }
    my $ClassObject = LoadClassByAlias( 'Address' );
    $ClassObject->deleteAttribute('L1');
    return 1;
}

sub Init {
    Cleanup();
    my $ClassObject = LoadClassByAlias( 'Address' );
    $ClassObject->insertAttribute({
        'Alias'  => 'L1',
        'Type'   => 'LocalizedString',
        'Package'=> 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute',
    });
    LoadRootObject()->insertChild( 'Address', {
        'Alias'   => 'O1',
        'Street'  => 'Leutragraben',
        'City'    => 'Jena',
        'Zipcode' => '07740',
        'Company' => 'epages',
    });
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
