/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.event.adjustonresize $Revision: 1.13.4.2 $

*/

dojo.provide('epages.event.adjustonresize');
dojo.require('epages.lang.hitch');
dojo.require('epages.browser');

epages.event.AdjustOnResize = function(/*DOMObject*/scrollbarElement, /* Object */ options) {
// summary: constructor for adjustonresize
// scrollbarElement  - dom node
// options 				   - object
// 		deltaWindowWidth  - integer or null
// 		deltaWindowHeight - integer or null
// 		callback 					- callback function

	// set element with scrollbar
	this.scrollbarElement=scrollbarElement;
	// set options
	this.setOptions(options);

	this.appendEvent();
};

epages.event.AdjustOnResize.prototype = {
	options						: undefined,
	scrollbarElement  : undefined,

	setOptions: function(/*Object*/ options) {
		if(this.options === undefined){
			this.options={
				deltaWindowHeight : undefined,
				deltaWindowWidth  : undefined,
				callback          : undefined,
				topicname         : 'AdjustOnResize'
			};
		}
		dojo.mixin(this.options, options);
		// update display
		this.resizeElement();

		dojo.addClass(document.getElementsByTagName("html")[0], "AdjustOnResize");
	},

	appendEvent: function() {
		// dojo.connect sometimes has problems with "onresize" on the window object
		// dojo.connect(window, 'onresize', epages.lang.hitch(this, function() { this.resizeElement()}));

		if(epages.Browser.engine=="MSIE"){          // MSIE version
			window.attachEvent('onresize', epages.lang.hitch(this, function() { this.resizeElement();}));
		} else {                                  // W3C version
			window.addEventListener("resize",epages.lang.hitch(this, function() { this.resizeElement();}),false);
		}
	},

	resizeElement: function() {
		var elementHeight = null;
		var elementWidth = null;

		//adjust height
		if(this.options.deltaWindowHeight != null) {
			var windowHeight = (window.innerHeight !== undefined) ? window.innerHeight : document.documentElement.clientHeight;  // height of the visible area in the window
			elementHeight=windowHeight-this.options.deltaWindowHeight;
			if(elementHeight>0) {
				this.scrollbarElement.style.height=elementHeight+'px';
			}
		}

		// adjust width
		if(this.options.deltaWindowWidth != null) {
			var windowWidth = (window.innerWidth !== undefined) ? window.innerWidth : document.documentElement.clientWidth;  // width of the visible area in the window
			elementWidth=windowWidth-this.options.deltaWindowWidth;
			if(elementWidth>0) {
				this.scrollbarElement.style.width=elementWidth+'px';
			}
		}

		dojo.publish(this.options.topicname+"/resize", [elementWidth, elementHeight]);

		if(this.options.callback !== undefined){
			if(typeof this.options.callback == 'function'){
				this.options.callback();
			}else{
				console.warn('option "callback" is not a function in epages.event.adjustonresize');
			}
		}
	}
};