/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.lang.color $Revision: 1.8.4.1 $

*/
dojo.provide('epages.lang.color');

epages.lang.color = {

	_hex:	function(c){
		c = parseInt(c).toString(16);
		return c.length < 2 ? "0" + c : c;
	},

	rgb2Hex: function(rgb){
		if (rgb.transparent == 1){
			return "transparent";
		}
		return '#' + this._hex(rgb.r) + this._hex(rgb.g) + this._hex(rgb.b);
	},

	hex2Rgb: function(hex) {
		if (hex == "transparent" || hex == "none"){
			return { r:255, g:255, b:255, transparent:1 };
		}
		hex = hex.toUpperCase();
		if(hex.charAt(0) == "#"){
			hex = hex.substring(1,hex.length);
		}
		if (hex.length == 6) {
			var rgb = {
				r: parseInt(hex.substring(0,2),16),
				g: parseInt(hex.substring(2,4),16),
				b: parseInt(hex.substring(4,6),16),
				transparent: 0
			};
		} else if (hex.length == 3) {
			var rgb = {
				r: parseInt(hex.substring(0,1) + hex.substring(0,1),16),
				g: parseInt(hex.substring(1,2) + hex.substring(1,2),16),
				b: parseInt(hex.substring(2,3) + hex.substring(2,3),16),
				transparent: 0
			};
		} else {
			//return white if no hex value was given
			var rgb = {
				r: 255,
				g: 255,
				b: 255,
				transparent: 0
			};
		}
		if(isNaN(rgb.r)){ rgb.r = 0; }
		if(isNaN(rgb.g)){ rgb.g = 0; }
		if(isNaN(rgb.b)){ rgb.b = 0; }

		return (rgb);
	},

	rgb2Hsb: function(rgb) {
		var min = new Number(Math.min(rgb.r, rgb.g));
		var max = new Number(Math.max(rgb.r, rgb.g));
		var sat = new Number(0);
		var lumin = new Number(0);
		min = Math.min(min, rgb.b);
		max = Math.max(max, rgb.b);
		var delta = (max - min);
		if (max != 0) {
			sat = delta / max;
			lumin = max / 255;
		}
		var hue = 0;
		if ( delta != 0 ) {
			if (rgb.r==max){
				hue = (rgb.g - rgb.b) / delta;
			}else if (rgb.g==max){
				hue = 2 + ((rgb.b - rgb.r) / delta);
			}else{
				hue = 4 + ((rgb.r - rgb.g) / delta);
			}
		}
		hue = (hue * 60);
		if (hue < 0){
			hue = hue + 360;
		}
		var tSat = (sat * 100);
		var tLum = (lumin * 100);

		return {s: Math.round(tSat), b: Math.round(tLum), h: Math.round(hue)};
	},

	hsb2Rgb: function(hsb) {
		var r = 0; var g = 0; var b = 0;
		var sat = hsb.s/100;
		var lum = hsb.b/100;
		//console.debug('sat ',sat,' lum ',lum,' hsb.h ',hsb.h);
		if (sat == 0) {
			r = lum;  g = r;  b = r;
		}
		else {
			var h = hsb.h/60;
			//if(h == 6) h = 0;
			var i = parseInt(h);
			var p = lum * (1.0 - sat);
			var q = lum * (1.0 - sat * (h - i));
			var t = lum * (1.0 - sat * (1- (h - i)));
			switch (i) {
				case 0: r = lum; g = t;   b = p;   break;
				case 1: r = q;   g = lum; b = p;   break;
				case 2: r = p;   g = lum; b = t;   break;
				case 3: r = p;   g = q;   b = lum; break;
				case 4: r = t;   g = p;   b = lum; break;
				case 5: r = lum; g = p;   b  = q;  break;
				case 6: r = lum; g = t;   b  = p;  break;
			}
		}
		return { r : (r*255), g : (g*255) , b : (b*255) };
	},

	cssString2Rgb: function(cssString) {
		return this.hex2Rgb(epages.validate.css.color2Hex(cssString));
	}

};