/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.localize $Revision: 1.15.4.4 $

*/
dojo.provide("epages.localize");
dojo.require('epages.epages');
dojo.require('epages.lang.hash');
dojo.require('epages.lang.array');
dojo.require("dojo.string");
dojo.require("dojo.date");
dojo.require("dojo.date.locale");

(function(){
	
epages.localize = {
	// summary: formats a number by using formatting chars from the user's locale
	formatNumber: function(/*Float*/value, /*Integer?*/decimalDigits) {
		var negativeString = (value < 0) ? '-' : '';
		var valueAbs = Math.abs(value);

		if (decimalDigits == null){
			decimalDigits = 2;
		}
		var precision = Math.pow(10, decimalDigits);
		var intPart = Math.floor(Math.round(valueAbs * precision) / precision);
		var ratPart = Math.round(Math.abs(valueAbs - intPart) * precision);
		if (ratPart != 0) {
			return negativeString +intPart + epages.vars.Locale.decimalSep + ratPart;
		} else {
			return negativeString+intPart;
		}
	},

	formatBytes: function(bytes, decimalDigits) {
		if (decimalDigits == null){
			decimalDigits = 2;
		}
		var unit = "Byte";
		if (bytes >= 1073741824) {
			// GB
			bytes /= 1073741824;
			unit = "GB";
		} else if (bytes >= 1048576) {
			// MB
			bytes /= 1048576;
			unit = "MB";
		} else if (bytes >= 1024) {
			// kB
			bytes /= 1024;
			unit = "kB";
		}
		return epages.localize.formatNumber(bytes, decimalDigits) + " " + unit;
	},

  // assign epages pattern elements to dojo pattern elements
  // see http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns for more
  // information
  // IMPORTANT! Only patterns that are supported by epages (existing Locales) are defined
	_patternCache: $H(),
	_patternMap: {
	  // -- YEAR -- //
 		// full year, 2007
		'%{ce_year}': 'yyyy',
 		// short year, 2 digits: 07
		'%y': 'yy',

		// -- MONTH -- //
 		// localized full month name: Oktober
		'%B': 'MMMM',
 		// localized month abbreviation: Okt
		'%b': 'MMM',
 		// month, 2 digits: 05 (may)
		'%m': 'MM',
 		// month, 5
		'%{month}': 'M',

		// -- DAY -- //
		// day, 2 digits, leading zero: '07'
		'%d': 'dd',
		// day, 2 digits, leading zero: '07'
		'%{day}': 'd',
 		// localized day name: Montag
		'%A': 'EEEE',

		// -- HOUR -- //
 		// hour, 24h, 2 digits, leading zero: '09'
		'%H': 'HH',
 		// hour, 24h
		'%{hour}': 'H',
 		// hour, 12h
		'%{hour_12}': 'h',

		// -- MINUTE -- //
 		// minutes, 2 digits, leading zero: '05'
		'%M': 'mm',

		// -- SECOND -- //
		// seconds, 2 digits, leading zero: '05'
		'%S': 'ss',

		// -- SPECIAL -- //
 		// AM or PM, upper case: AM
		'%p': 'a',
 		// time zone, '+0100'
    	'%{time_zone_long_name}': 'ZZ'
	},


	getDojoDateTimePattern: function(epagesPattern) {
		// summary: converts an epages date pattern to a Dojo date pattern.
	  // note: this function does not support all declared specifiers
		if (epagesPattern == null){
			epagesPattern = epages.vars.Locale.dateFormat;
		}
		if (this._patternCache.existsKey(epagesPattern)){
			return this._patternCache.get(epagesPattern); //String
		}
	  var dojoPattern = epagesPattern;
		// relace elements in string with values
		var elementPatterns = $A($A($H(epages.localize._patternMap).keys()).sort(function(a, b) { return b.length - a.length; }));

		elementPatterns.each(function(element) {
		  var rx = new RegExp(element, 'g');
		  dojoPattern = dojoPattern.replace(rx, function() {
		    return epages.localize._patternMap[element];
		  });
		});
		return dojoPattern; //String
	},

	formatDateTime: function(/*Date*/date, /*String?*/epagesPattern) {
	  // summary: replaces the pattern specifiers with real values
		if (date == null){
			return '';
		}
		if (epagesPattern == null){
			epagesPattern = epages.vars.Locale.dateFormat;
		}
		var d = dojo.date.locale.format(date, {
		  datePattern: this.getDojoDateTimePattern(epagesPattern),
		  timePattern: ' '
		});
		// trim last to chars --> spaces
		return d.substring(0, d.length - 2);
	},

	parseDateTime: function (/*String*/ dateString, /*String*/ epagesPattern) {
		if (epagesPattern == null){
			epagesPattern = epages.vars.Locale.dateFormat;
		}
		return dojo.date.locale.parse('~ ' + dateString, {
			selector: 'timeOnly',
			timePattern: this.getDojoDateTimePattern(epagesPattern),
			datePattern: '~'
		});
	},

	parseNumber: function(/*String*/ numberString){
		numberString = numberString.replace(epages.vars.Locale.thousandSep,'');
		numberString = numberString.replace(epages.vars.Locale.decimalSep,'.');
		return parseFloat(numberString);

	},
	getFormat: function(type) {
		return _format[type];
	},
	getPattern: function(type) {
		return _pattern[type];
	},
	getSeparator: function(type) {
		return _separator[type];
	},
	date2obj: function( input , type ) {
		return dojo.date.locale.parse( 
					input, 
					{ 
						selector: 		'date', 
						datePattern: 	_format[type], 
						timePattern: 	''
					}
				);
	},
	obj2date: function( input , type ) {
		return dojo.date.locale.format( 
					input, 
					{ 
						selector: 		'date', 
						datePattern: 	_format[type], 
						timePattern: 	'' 
					}
				);
	},
	date2unix: function( input , type ) {
		var ret = epages.localize.date2obj( input , type );
		return ret ? ( ret.getTime() / 1000 ) : null;
	},
	unix2date: function( input , type ) {
		return epages.localize.obj2date( new Date(input*1000) , type );
	},
	str2number: function( input ) {
		var ret = parseFloat( 
						(input+'')
							.replace( new RegExp( dojo.regexp.escapeString('[^0-9-' +_separator.decimal + ']', '^([]{})$'), 'g') , '' )
							.replace( new RegExp( dojo.regexp.escapeString('[' + _separator.decimal + ']', 	'^([]{})$'), 'g') , '.' ) 
					);
					
		return isNaN(ret) ? null : ret;
	},
	number2str: function( input ) {
		return (input+'').replace( /\./g , _separator.decimal );
	}
};

// private vars for use in epages.localize
var _separator = {
		decimal: 		epages.vars.Locale.decimalSep,
		thousand: 		epages.vars.Locale.thousandSep,
		money: 			epages.vars.Locale.moneySymbol
	},
	_format = {
		date:			epages.localize.getDojoDateTimePattern( epages.vars.Locale.dateFormat ),
		datetime:		epages.localize.getDojoDateTimePattern( epages.vars.Locale.datetimeFormat ),
		month:			'MM (yyyy)',
		time:			epages.localize.getDojoDateTimePattern( epages.vars.Locale.timeFormat ),
		week:			'w (yyyy)'
	},
	_pattern = {
		space: 			'\ \n\r\t\
',
		alpha: 			'A-Za-z',
		digit: 			'0-9',
		date:			dojo.date.locale.regexp({ selector:'date',	datePattern:_format.date,		timePattern:'' }),
		datetime:		dojo.date.locale.regexp({ selector:'date',	datePattern:_format.datetime,	timePattern:'' }),
		month: 			dojo.date.locale.regexp({ selector:'date',	datePattern:_format.month,		timePattern:'' }),
		time:			dojo.date.locale.regexp({ selector:'date',	datePattern:_format.time,		timePattern:'' }),
		week: 			dojo.date.locale.regexp({ selector:'date',	datePattern:_format.week,		timePattern:'' }),
		email:			dojo.regexp.escapeString('[A-Za-z0-9._-]{1,}@[A-Za-z0-9._-]{2,}[A-Za-z]{2,4}', '([]{})'),
		url: 			'([A-Z]:\\[^/:\*\?<>\|]+\.\w{2,6})|(\\{2}[^/:\*\?<>\|]+\.\w{2,6})',
		tel: 			'[0-9\/\+\-\.\ ]{0,}',
		number:			dojo.regexp.escapeString('-? ?[0-9'+_separator.thousand+']+['+_separator.decimal+']?[0-9]*', '([]{}+?*)'),
		integer:		'-?[0-9]*',
		money:			dojo.regexp.escapeString('(['+_separator.money+']? ?-? ?[0-9'+_separator.thousand+']{1,}['+_separator.decimal+']?[0-9]{0,2}|-? ?[0-9'+_separator.thousand+']{1,}['+_separator.decimal+']?[0-9]{0,2} ?['+_separator.money+']?)', '([|]{}?)')
	};
		
})();