/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.storage $Revision: 1.3.2.8 $
*/

dojo.provide("epages.storage");
dojo.require("epages.epages");
dojo.require("dojo._base.json");

(function(dj){
		  
	var trim = function(string) {
		return (string+'').replace(/[\ \r\n\t]*(.*)[\ \r\n\t]*/,"$1");
	};	
	
	var domain = 			'; domain=' + document.domain,
		path = 				'; path=/',
		siteId = 			'SiteID::'+epages.vars.SiteID+'::',
		
		isLocalStorage = 	window.localStorage && window.localStorage.setItem ? true : false,
		isSessionStorage = 	window.sessionStorage && window.sessionStorage.setItem ? true : false,
		
		setCookie = function(name, value, expires) {
			if( typeof expires == 'number' ) {
				var date = new Date();
					date.setDate( expires );
				expires = '; expires=' + date.toUTCString();
			}
			else{
				expires = '';
			}
			document.cookie = [name, '=', dj.toJson(value), expires, path, domain].join('');
			return value;
		},
		
		getCookie = function(name) {
			var value = null;
			if (document.cookie && document.cookie != '') {
				var cookies = document.cookie.split(';');
				for (var i = 0,l=cookies.length; i < l; i++) {
					var cookie = trim(cookies[i]);
					// Does this cookie string begin with the name we want?
					if (cookie.substring(0, name.length + 1) == (name + '=')) { 
						value = dj.fromJson(cookie.substring(name.length + 1));
						break;
					}
				}
			}
			return value;
		},
		
		removeCookie = function( name ) {
			return setCookie( name, null, -365);
		},
		
		clearCookie = function( prefix ) {
			if (document.cookie && document.cookie != '') {
				var cookies = document.cookie.split(';');
				for (var i = 0,l=cookies.length; i < l; i++) {
					var cookie = trim(cookies[i]);
					if( cookie.match(new RegExp('^'+prefix,'g')) ) {
						removeCookie(cookie.substr(0,cookie.indexOf('=')));
					}
				}
			}
			return null;
		};
		
	//simulate the storage fallback:
	//isLocalStorage = false;
	//isSessionStorage = false;
		
	
	dj.mixin(epages,{
		localStorage: {
			setItem: function( name, value ) {
				return isLocalStorage ? window.localStorage.setItem( siteId+name, dj.toJson(value) ) : setCookie( 'localStorage::'+siteId+name, value, 365 );
			},
			getItem: function( name ) {
				if( isSessionStorage ) {
					var ret = window.localStorage.getItem( siteId+name );
					return ret ? dj.fromJson( ret ) : ret;
				}
				else {
					return  getCookie( 'localStorage::'+siteId+name );
				}
			},
			removeItem: function( name ) {
				return isLocalStorage ? window.localStorage.removeItem( siteId+name ) : removeCookie( 'localStorage::'+siteId+name );
			},
			clear: function( name ) {
				return isLocalStorage ? window.localStorage.clear() : clearCookie('localStorage::'+siteId);
			}
		},
		sessionStorage: {
			setItem: function( name, value ) {
				return isSessionStorage ? window.sessionStorage.setItem( siteId+name, dj.toJson(value) ) : setCookie( 'sessionStorage::'+siteId+name, value );
			},
			getItem: function( name ) {
				if( isSessionStorage ) {
					var ret = window.sessionStorage.getItem( siteId+name );
					return ret ? dj.fromJson( ret ) : ret;
				}
				else {
					return  getCookie( 'sessionStorage::'+siteId+name );
				}
			},
			removeItem: function( name ) {
				return isSessionStorage ? window.sessionStorage.removeItem( siteId+name ) : removeCookie( 'sessionStorage::'+siteId+name );
			},
			clear: function( name ) {
				return isSessionStorage ? window.sessionStorage.clear() : clearCookie('sessionStorage::'+siteId);
			}
		}
	});

})(dojo);