/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.template $Revision: 1.17.4.2 $

*/
dojo.provide('epages.template');
dojo.require('epages.epages');
dojo.require('epages.html.element');
dojo.require('epages.lang.hash');

$TMPL = /*epages.Template*/function (/*string*/template,/*hash*/ vars, /*boolean?*/ encodeVars, /*boolean?*/ enableRegExVars) {
	if(encodeVars === undefined){
		encodeVars=true;
	}
	if(enableRegExVars === undefined){	// default reg ex vars are allowed
		enableRegExVars=true;
	}

	return new epages.Template({
		template: template,
		vars: vars,
		encodeVars: encodeVars,
		enableRegExVars : enableRegExVars
	});
};

epages.Template = function(options) {
	this.setOptions(options);

	// get Template from DOM
	if (this.options.template === undefined && this.options.templateDom != undefined) {
		if (typeof this.options.templateDom != 'object'){
			this.options.templateDom = $(this.options.templateDom);
		}
		this.options.template = $E(this.options.templateDom).firstComment();
	}
};

epages.Template.prototype = {
	options : undefined,

	setOptions: function(options) {
		this.options = {
			encodeVars  : true,
			enableRegExVars: false,
			templateDom : undefined,
			template    : undefined,
			vars        : undefined,
			addTemplate : true
		};
		dojo.mixin(this.options, options);
	},

	setTemplate: function (template) {
		this.options.template = template;
	},

	encodeXml: function(string) {
		if (string !== undefined) {
			if (typeof string == "number"){
				string = string.toString();
			}
			string = string
				.replace(/&/g,"&amp;")     // qoute &
				.replace(/"/g,"&quot;")    // qoute "
				.replace(/>/g,"&gt;")      // qoute >
				.replace(/</g,"&lt;");     // qoute <
		}
		return string;
	},

	maskRegExVars: function(string) {
		if (string !== undefined) {
			if (typeof string == "number"){
				string = string.toString();
			}
			if (typeof string == "string"){
				string = string.replace(/\$/g, "\\$");     // mask $
			}
		}
		return string;
	},

	unmaskRegExVars: function(string) {
		if (string !== undefined) {
			if (typeof string == "number"){
				string = string.toString();
			}
			if (typeof string == "string"){
				string = string.replace(/\\\$/g, "$");     // unmask $
			}
		}
		return string;
	},

	processHash: function(vars) {
		// declare some vars for iterator function
		if (vars.each === undefined){
			vars = $H(vars);
		}
		var template=this.options.template;
		if (template === undefined) {
			console.warn('epages.Template: template is empty');
			return '';
		}

		var me=this;
		// insert hash vars into template
		var iterator= function(pair) {
			var tempValue = pair.value;
			var encValue = me.options.encodeVars ? me.encodeXml(tempValue) : tempValue;
			// insert string
			encValue = me.options.enableRegExVars ? encValue : me.maskRegExVars(encValue);

			var templateRegExp = new RegExp('\\(%' + pair.key + '%\\)', "g" );
			template = template.replace( templateRegExp, encValue );
			template = me.options.enableRegExVars ? template : me.unmaskRegExVars(template);
		};
		// call iterator for each key/value pair
		vars.each(iterator);
		return template;
	},

	processLoop: function(loop) {
		var content = '';
		var me=this;
		loop.each(function (el) {
			content += me.processHash(el);
		});
		return content;
	},

	generateOutput: function () {
		return this.processHash(this.options.vars);
	},

	insertTemplate: function (element) {
		if (!this.options.addTemplate){
			return;
		}
		var comment = document.createElement("COMMENT");
		comment.data = this.options.template;
		// element.insertAdjacentElement("afterBegin",comment);
		if (element.firstChild === undefined){
			element.insertChild(comment);
		}else{
			element.insertBefore(comment, element.firstChild);
		}
		return;
	},

	insertLoop: function (loop, targetDom) {
		if (targetDom === undefined){
			targetDom = this.options.templateDom;
		}
		if (typeof this.options.targetDom != 'object'){
			targetDom = $(targetDom);
		}
		targetDom.innerHTML = this.processLoop(loop);
		this.insertTemplate(targetDom);
	},

	insert: function(targetDom) {
		if (targetDom === undefined){
			targetDom = this.options.templateDom;
		}
		if (typeof this.options.targetDom != 'object'){
			targetDom = $(targetDom);
		}
		targetDom.innerHTML = this.generateOutput();
		this.processTemplate(targetDom);
	}
};
