/*  EPages Test JavaScript module, $Revision: 1.8.2.1 $
 *  (c) 2006 ePages GmbH
/*--------------------------------------------------------------------------*/
dojo.provide("epages.test");
dojo.require("epages.epages");

epages.Test = function(options) {
    options = options || {};
    this.planned = options.planned || 1;
    this.correct = 0;
    this.failed  = 0;
    this.current = 0;
    this.failedList = new Array();
    this.outputElement = options.outputElement || undefined;
};

epages.Test.prototype = {
  planned:  0,
  correct:  0,
  failed:   0,
  current:  0,
  failedList:  undefined,

  setPlanned: function(planned) {
    this.planned = planned;
  },

  ok: function(isCorrect, testDescription) {
    var returnText = '';
    this.current++;
    if (isCorrect) {
      this.correct++;
    } else {
      this.failed++;
      returnText = 'not ';
      this.failedList.push(this.current);
    }
    returnText += 'ok ' + (this.current) + ' - ' + testDescription;
    if (this.outputElement !== undefined){
      this.outputElement.innerHTML +='<div class="testLine">' + returnText + '</div>';
    }
    return returnText;
  },

  test: function( func, description ) {
    var returnValue = 0;
    try {
      returnValue = func();
      this.ok(returnValue, description);

    } catch (e) {
      this.ok(returnValue, description + ' (crashed)');
    }
    return returnValue;
  },

  begin: function () {
    return '1..'+ this.planned;
  },

  summary: function () {
    var returnText = '';
    if (this.failed > 0){
      returnText += '# Looks like you failed ' + this.failed + ' test of ' + this.planned + '.\n';
    }
    var runTests = this.failed + this.correct;
    if (runTests != this.planned) {
      var only = '';
      if (runTests < this.planned){
        only = 'only';
      }
      returnText += '# Looks like you planned ' + this.planned + ' tests but ' + only + ' ran ' + runTests + '.\n';
    }
    return returnText;
  },

  fillResults: function ( options ) {
    if (options.planned    !== undefined){ 
    	options.planned.innerHTML    = this.planned; 
    }
    if (options.correct    !== undefined){ 
    	options.correct.innerHTML    = this.correct; 
    }
    if (options.failed     !== undefined){ 
    	options.failed.innerHTML     = this.failed; 
    }
    if (options.failedList !== undefined){ 
    	options.failedList.innerHTML = (this.failed ? this.failedList.join(',') : 'no failed tests'); 
    }
    if (options.summary    !== undefined){ 
    	options.summary.innerHTML     = this.summary(); 
    }
  }

};

// alert('load_Test_finish');
