dojo.provide("epages.widget.Calendar");
dojo.require("epages.widget");
dojo.require('dijit.Calendar');
dojo.require('epages.lang.date');
dojo.require('epages.widget.Tooltip');
dojo.require('epages.io.translation');

dojo.declare("epages.widget.Calendar", [dijit.Calendar], {
		/**
		 * public properties
		 */
		firstAvailableDate : null,
		lastAvailableDate  : null,

		/**
		 * private properties
		 */
		templateType       : "default",
		defaultTemplateType: "default", // read-only!
		templatePath       : dojo.moduleUrl("epages.widget", "templates/Calendar.html"), // need template for weeknumber
		_templateDirectory : dojo.moduleUrl("epages.widget", "templates"),
		widgetsInTemplate : true,
		_templateMapping : {
			'default' : 'Calendar.html',
			'flat'    : 'CalendarFlat.html'
		},
		_yearTip: '',

		postMixInProperties: function() {
			this.inherited("postMixInProperties", arguments);
			if (this._templateMapping !== undefined &&  this.templateType !== undefined &&  this._templateDirectory !== undefined) {
				if(this._templateMapping[this.templateType]!==undefined) {
					// check if set to widget's default using its prototype value
					if(this.templateType != this.defaultTemplateType) {
						var tempPath = '';
						tempPath = this._templateDirectory;
						tempPath.path += '/'+this._templateMapping[this.templateType];
						tempPath.uri += '/'+this._templateMapping[this.templateType];
						this.templatePath = tempPath;
						this.templateString = null;
					}
				} else {
					console.warn('Template type: "'+ this.templateType +'" does not exist in this widget - use default. ('+this.declaredClass+')');
				}
			}
			this._dictionary = new epages.io.Translation(dojo.moduleUrl('epages.widget', "templates/translation"), "auto");
			this._yearTip = this._dictionary.get('YearTip');


		},

		postCreate: function() {
			var d = this.value;
			this.inherited("postCreate", arguments);
			this.attr('value',null);
			this.attr('value',d);

		},

		_populateGrid: function(){
			epages.widget.Calendar.superclass._populateGrid.apply(this);
			dojo.query(".dijitCalendarWeekLabel", this.domNode).forEach(function(label, i) {
				if (i) {
					var node = label.parentNode.nextSibling;
					while (node != null && node.nodeName == '#text'){
						node = node.nextSibling;
					}
					this._setText(label, epages.lang.date.getWeekOfYear(new Date(node.dijitDateValue), dojo.cldr.supplemental.getFirstDayOfWeek(this.lang)));
				}
			}, this);
		},

		_dateNode: function (node) {
			while(node && !node.dijitDateValue){ node = node.parentNode; }
			return node;
		},

		// disallow clicks on calendar weeks
		_onDayClick: function(/*Event*/evt){
			var node = evt.target;
			if(dojo.hasClass(node, "dijitCalendarDateLabel")) {
				epages.widget.Calendar.superclass._onDayClick.apply(this, arguments);
			} else {
				dojo.stopEvent(evt);
			}
		},

		isDisabledDate: function(/*Date*/ date, /*String?*/ locale) {
			date.setHours(0,0,0,0);
			// check if a date is between a the available period
			return (this.lastAvailableDate && date > this.lastAvailableDate) ||
						 (this.firstAvailableDate && date < this.firstAvailableDate);
		}
});
