/*
	Copyright (c) 2006-2009, epages Software GmbH
	All Rights Reserved.

	epages.widget.DragDropElementlist $Revision: 1.2.4.3 $
*/

dojo.provide("epages.widget.DndElementlist");
dojo.require("epages.widget.Elementlist");

dojo.declare("epages.widget.DndElementlist", [epages.widget.Elementlist], {

		avatarTextSelected: "",
		dndAvatarNode : undefined,
		dndAvatarContentNode: undefined,
		maxAvatarElements: "6",

		_startDragging : false,		// indicates that dragging is triggered (mousedown on selected element)
		_isDragging    : false,		// indicates that a user really moves items (mousemove)
		_stopDescelect : false,
		_isAvatarVisible: false,

		postMixInProperties: function() {
			this.inherited("postMixInProperties", arguments);
			// add default event names
			this.eventNamesDefault.drop='drop';  // dnd drag
			this.eventNamesDefault.move='move';	// dnd move
			this.eventNamesDefault.startDragging='startDragging';	// dnd start dragging
			this.eventNamesDefault.stopDnd='stopDnd';	// dnd stop dnd - dnd end / dnd cancelled

			this.maxAvatarElements=parseInt(this.maxAvatarElements);
			if(this.maxAvatarElements === NaN) {
				console.warn("Attribute maxAvatarElements is not a number in "+this.declaredClass+". Default value is used.");
				this.maxAvatarElements = 6;
			}
		},

		postCreate: function() {
			this.inherited("postCreate", arguments);

			this.dndAvatarNode = document.createElement("div");
			dojo.addClass(this.dndAvatarNode, "HideElement DndAvatar");
			this.dndAvatarContentNode = document.createElement("div");
			this.dndAvatarContentNode.className="Inner";
			this.dndAvatarNode.appendChild(this.dndAvatarContentNode);
			document.body.appendChild(this.dndAvatarNode);

			this._appendEvents();
		},

		_appendEvents: function() {
			this.connect(window, "onkeypress", "onKeypress");
			this.connect(document.body, "onkeypress", "onKeypress");
			this.connect(document, 'onmousemove', '_onmousemove');
			this.connect(document, 'onmouseup', '_onmouseup');
		},

		onKeypress :function(evt) {
			if(evt.keyCode == dojo.keys.ESCAPE) {
				this._stopDrop();
			}
		},

		_itemOnMouseDown: function(itemWdg, isSel, evt) {
			this._startDragging=true;
			var itemNames = (this.avatarTextSelected != "") ? "<b>"+this.avatarTextSelected+"</b><br />" : "";
			var items = this.selectedNodes();
			var el = items[0];
			var i = 0;
			while(el && i<this.maxAvatarElements) {
				itemNames+=el.title+"<br />";
				i++;
				el = items[i];
			}

			if(items.length > this.maxAvatarElements) {
				itemNames+="...<br />";
			}

			this.dndAvatarContentNode.innerHTML = itemNames;

			//avoid text selection
			document.body.onselectstart = function() { return false; };
			this.domNode.style.MozUserSelect = "none";
		},

		_onmousemove: function(evt) {
			if(this._startDragging && !this._isDragging) {
				var pos = this._getMousePosition(evt);
				dojo.publish(this.eventNames.startDragging, [{
					x: pos.x,
					y: pos.y,
					items: this.selectedNodes(),
					handler: this.id
				}]);	// real start dragging starts with moving not onmousedown
			}
			if(this._startDragging) {
				this._isDragging=true;
				var pos = this._getMousePosition(evt);
				// avatar
				this.dndAvatarNode.style.left = (pos.x + 10)+"px";
				this.dndAvatarNode.style.top= (pos.y +10)+"px";
				if(!this._isAvatarVisible){
					dojo.removeClass(this.dndAvatarNode, "HideElement");
				}

				// publish event
				dojo.publish(this.eventNames.move, [{
					x: pos.x,
					y: pos.y,
					items: this.selectedNodes(),
					handler: this.id
				}]);
			}
		},

		_onmouseup: function(evt) {
			if(this._isDragging) {
				dojo.addClass(document.body, "DropWait");
				var pos = this._getMousePosition(evt);

				dojo.publish(this.eventNames.drop, [{
					x: pos.x,
					y: pos.y,
					items: this.selectedNodes(),
					handler: this.id
				}]);
			}
			this._stopDrop();
		},

		_stopDrop: function() {
			this._startDragging=false;
			if(this._isDragging) {
				this._isDragging=false;

				// hide avatar
				dojo.addClass(this.dndAvatarNode, "HideElement");
				dojo.publish(this.eventNames.stopDnd, [{
					x: undefined,
					y: undefined,
					items: this.selectedNodes(),
					handler: this.id
				}]);
				dojo.removeClass(document.body, "DropWait");
			}

			//allow text selection
			document.body.onselectstart = function(){};
			// this.domNode.style.MozUserSelect = ""; --> this somehow harms drop in tree
		},

		_getMousePosition: function(evt) {
			// summary: returns mouse position on screen
			if(epages.Browser.engine=="MSIE") {
				var x= evt.clientX;
				var y= evt.clientY;
			} else {
				var x= evt.pageX;
				var y= evt.pageY;
			}
			return { x:x, y:y };
		},

		dndSelectItem: function(item, isSelected, evt) {
			if(!isSelected) {
				this.doSelectItem(item, isSelected, evt);
				this._stopDescelect = true;
			}
		},

		dndDeSelectItem: function(item, isSelected, evt) {
			if(isSelected && !this._stopDescelect && !this.isDragging) {
				this.doSelectItem(item, isSelected, evt);
			}
			this._stopDescelect = false;
		}

	}
);

