/*
	Copyright (c) 2010, epages Software GmbH
	All Rights Reserved.

	epages.widget.FormElements $Revision: 1.1.2.4 $

*/

(function(dj){

	dj.provide('epages.widget.FormElements');
	
	dj.require('dojo.date.locale');
	dj.require('dojo.regexp');
	dj.require('epages.epages');
	dj.require('epages.localize');
	dj.require('epages.browser');
	dj.require('epages.uploader');
	dj.require('epages.event');
	dj.require('epages.widget.Datetimeinput');
	
	dj.declare('epages.widget.FormBase', [dijit._Widget], {
		
	// VARS
		wrapNode		: null,				// node of wrap which wrapped the domNode
		formNode		: null,				// node of parent form
		_type			: 'text',			// type of formElement
		_attr			: {},				// attributes of formElement
		_handles		: null,				// array of handles which will connect on this._connectHandles
	
	// CREATE
		postCreate: function() {
			this.inherited('postCreate', arguments);
			
			this._attr = {};
			this._handles = [];
			
			// get type
			if( this.domNode.nodeName.match(/^(TEXTAREA|SELECT)$/) ) {
				this._attr.type = this.domNode.nodeName.toLowerCase();
			}
			else {
				this._attr.type = dj.attr(this.domNode,'type');
			}
			if( this._attr.type=='datetime-local' ) {
				this._attr.type = 'datetime';
			}
			if( !this._attr.type ) {
				var match = (this.domNode.outerHTML).match(/[<\s]{1}type="?([A-Za-z]+)"?[>\s\/]{1}/);
				this._attr.type = match ? match[1] : 'text';
			}
			
			// get attributes
			var ATTR = ['autofocus','form','formaction','formmethod','formnovalidate','id','required','pattern','min','max','name','accept','checked','disabled','list','maxlength','multiple','placeholder','step','value'];
			
			for( var i=0,l=ATTR.length ; i<l ; i++ ) {
				var name = ATTR[i];
				var value = dj.attr(this.domNode,name);
				// parse min, max & step to number
				if( name.match(/^(max|min|step)$/) ) {
					value = epages.localize.str2number(value);
					if( value!==null ) {
						this._attr[name] = value
					};
				}
				// is defined
				else if( value !== undefined ) { 
					this._attr[name] = value;
				};
			}
			this._attr.id = this.id
			this._attr.valide = dj.hasClass(this.domNode,'DialogError') ? false : true;
			
			// replace domNode for type compatibility
			if( this._attr.type.match(/^(color|date|datetime|email|month|number|search|tel|time|url|week)$/) ) {
				var node = dj.clone(this.domNode);
					if( !(epages.Browser.engine=='MSIE' && epages.Browser.version<=8) ) {
						dj.attr( node , 'type' , 'text' );
					}
					dj.place( node , this.domNode , 'after' );
					node.parentNode.removeChild(this.domNode);
				this.domNode = node;
			}
			
			// wrap domNode
			this.wrapNode = dj.create('label',{
				'class': 'EpForm ' + this._attr.type.substr(0,1).toUpperCase() + this._attr.type.substr(1) + ' ' + dj.attr(this.domNode,'class')
			},this.domNode,'after');
			this.wrapNode.appendChild(this.domNode);
			
			// set formWidget
			var formNode = (this._attr.form ? dj.query('form[name='+this._attr.form+']') : [this.domNode.form])[0]; 
			if( !formNode ) {
				console.warn('formElement (widgetid:' + this.id + ') is not inside a form');
				formNode = dj.create('form',{},this.wrapNode,'after');
				formNode.appendChild(this.wrapNode);
			}
			this.formWdgt = $$( dj.attr(formNode,'widgetid') );
			if( !this.formWdgt ) {
				this.formWdgt = new epages.widget.Form( {}, formNode );
			}
			
			// fix to disable validation by browser
			dj.removeAttr( this.domNode, 'formnovalidate' );
			dj.removeAttr( this.domNode, 'formmethod' );
			dj.removeAttr( this.domNode, 'formaction' );
			dj.removeAttr( this.domNode, 'max' );
			dj.removeAttr( this.domNode, 'min' );
			dj.removeAttr( this.domNode, 'pattern' );
			dj.removeAttr( this.domNode, 'required' );
			dj.removeAttr( this.domNode, 'step' );
			
			// add this formElement to his form
			this.formWdgt.addElem( this );
			
			// set focus
			if( this._attr.autofocus == true ) {
				this._domNode.focus();
			}
		},
		// init the readonly and disbaled status
		postAfterCreate: function() {
			this.attr( 'readonly', this._attr.readonly || false );
			this.attr( 'disabled', this._attr.disabled || false );
		},
	
	// METHODS
		attr: function( name, value ) {
			if( typeof name == 'object' ) {
				for( var i in name ) {
					this.attr( i, name[i] );
				}
			}
			else {
				var methodName = '_attr'+name.substr(0,1).toUpperCase()+name.substr(1);
				if( value===undefined ) {
					return this[methodName] ? this[methodName]() : this._attr[name];
				}
				else {
					return this._attr[name] = this[methodName] ? this[methodName](value) : value;
				}
			}
		},	
		_attrMin: function( value ) {
			return value ? epages.localize.str2number(value) : this._attr.min;
		},	
		_attrMax: function( value ) {
			return value ? epages.localize.str2number(value) : this._attr.max;
		},	
		_attrStep: function( value ) {
			return value ? epages.localize.str2number(value) : this._attr.step;
		},	
		_attrType: function( value ) {
			// value will be ignored, type can not modify
			return this._attr.type;
		},
		// set/get formElement disabled status
		_attrForm: function( value ) {
			if( value || value=='' ) {
				if( value=='' ) {
					var formNodeList = [this.domNodeForm];
				}
				else {
					var formNodeList = dj.query('form[name='+value+']') || [];
				}
				if( formNodeList[0] ) {
					this.formWdgt.removeElem( this.id );
					this.formWdgt = $$( dj.attr(formNodeList[0],'widgetid') );
					if( !formWdgt ) {
						this.formWdgt = new epages.widget.Form( {}, formNodeList[0] );
						this.formWdgt.addElem( this );
					}
					this._attr.form = value;
				}
				return this._attr.form || '';
			}
			else {
				return this._attr.form;
			}
		},
		// set/get formElement disabled status
		_attrDisabled: function( bool ) {
			if( bool===true ) {
				dj.attr( this.domNode , 'disabled' , true );
				dj.addClass( this.wrapNode , 'Disabled' );
				
				this._disconnectHandles();
				return bool;
			}
			else if( bool===false ) {
				dj.removeAttr( this.domNode , 'disabled' );
				dj.removeClass( this.wrapNode , 'Disabled' );
				
				if( !this._attr.readonly ) {
					this._disconnectHandles();
					this._connectHandles();
				}
				return bool;
			}
			else {
				return this._attr.disabled || false;
			}
		},
		// set/get formElement readonly status
		_attrReadonly: function( bool ) {
			if( bool===true ) {
				dj.attr( this.domNode , 'readonly', true );
				dj.addClass( this.wrapNode , 'Readonly' );
				
				this._disconnectHandles();
				return bool;
			}
			else if( bool===false )  {
				dj.removeAttr( this.domNode , 'readonly' );
				dj.removeClass( this.wrapNode , 'Readonly' );
				
				if( !this.attr('disabled') ) {
					this._disconnectHandles();
					this._connectHandles();
				}
				return bool;
			}
			else {
				return this._attr.readonly || false;
			}
		},
		// connect all handles
		_connectHandles: function() {
			var _handles = this._handles;
			for( var i=0,l=_handles.length ; i<l ; i++ ) {
				this._connects.push( dj.connect( _handles[i][0], _handles[i][1], this, _handles[i][2] ) );
			}
		},
		// disconnect all connected handles
		_disconnectHandles: function() {
			dj.forEach( this._connects, dj.disconnect );
		},
		// replace pseudo pattern with regexp pattern
		_replacePattern: function() {
			if( this._attr.pattern ) {
				this._attr.pattern = this._attr.pattern.replace(/(\[\:[a-z\-]+\:\])/,function( t, s ) {
					return epages.localize.getPattern( s.substr(2,s.length-4) );
				} );
			}
		},
		// validate formElement
		validate: function() {
			var valide = true;
			
			if( !this.domNode.novalidate ) {
				valide = this._validate();
				if( valide ) {	
					dj.removeClass( this.domNode , 'DialogError' )
					dj.removeClass( this.wrapNode , 'DialogError' )
				}
				else {
					dj.addClass( this.domNode , 'DialogError' )
					dj.addClass( this.wrapNode , 'DialogError' )
				};
				this.attr('valide',valide);	
				this.formWdgt.change();
			}
			else {
				this.attr('valide',valide);
			}	
			
			return valide;
		},
		// placeholder for validate function
		_validate: function() {
			return true;
		},
		// destroy a widget
		destroy: function() {
			this.inherited('destroy',arguments);
			
			this.formWdgt.removeElem( this.id );
		}
		
	});
	
	
// WIDGET FormButton
	dj.declare('epages.widget.FormButton', [epages.widget.FormBase], {
	
	// CREATE
		postCreate: function() {
			this.inherited('postCreate', arguments);
			
			if( this._attr.type.match(/^(reset|submit)$/) ) {
				this._handles.push([ this.domNode, 'click', this._formAction ]);
			}
			
			this.postAfterCreate();
		},
		
	// METHODS	
		// on click action send to the form widget
		_formAction: function(event) {
			event.preventDefault();
			epages.event.fire( this.formWdgt.domNode, this._attr.type );
		}
	
	});
	
	
// WIDGET FormFile
	dj.declare('epages.widget.FormFile', [epages.widget.FormBase], {
    
    buttonNode: null,
    
	// CREATE
		postCreate: function() {
			this.inherited('postCreate', arguments);
			
			if( this._attr.formaction ) {
				this._handles.push([ this.domNode, 'change', this.uploader ]);
			}
      
      this.buttonNode = dj.create('span',{
          'class':''
      },this.domNode,'after');
      this.buttonNode.innerHTML = 'file upload...';
      
			this.postAfterCreate();
		},
		
	// METHODS	
		// open the uploader on change if attr formaction is set
		uploader: function() {
			if( this._attr.formaction ) {
				epages.Uploader.directOpen( this.domNode, {
					multiple : 		this.attr('multiple') || false,
					action : 		this.attr('formaction'),
					accept : 		this.attr('accept') || '*/*',
					acceptGroup : 	null,
					reloadPage : 	this.attr('formmethod')=='_reload' ? true : false,
					id : 			this.attr('id')
				});
			}
		}
	
	});
	
	
// WIDGET FormSelect
	dj.declare('epages.widget.FormSelect', [epages.widget.FormBase], {
	
	// CREATE
		postCreate: function() {
			this.inherited('postCreate', arguments);
			
			this._handles.push([ this.domNode, 'change', this.validate ]);
			
			this.postAfterCreate();
		},
		
	// METHODS	
		// set/get formElement required status (required cann't be true for type select
		_attrRequired: function() {
			this._attr.required = false;
			return false;
		}
	
	});
	
	
// WIDGET FormText
	dj.declare('epages.widget.FormText', [epages.widget.FormBase], {
	
	// CREATE
		postCreate: function() {
			this.inherited('postCreate', arguments);
			
			this._handles.push([ this.domNode 		, 'keyup' 		, this.validate ]);
			this._handles.push([ this.domNode 		, 'blur' 		, this.validate ]);
			
			// replace domNode for type compatibility
			if( this._attr.type.match(/^(email|tel|url)$/) ) {
				this._attr.pattern = epages.localize.getPattern( this._attr.type );
			}
			
			this.postAfterCreate();
		},
		
	// METHODS	
		_validate: function() {
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;	
			
			if( value=='' && !this._attr.required ) {
				valide = true;
			}
			if( value=='' && this._attr.required ) {
				valide = false;
			}
			else if( this._attr.pattern && value.match(new RegExp('^'+this._attr.pattern+'$')) ) {
				valide = true;
			}
			else {
				valide = true;
			};
			return valide;
		}
	
	});
	
	
// WIDGET FormCheck
	dj.declare('epages.widget.FormCheck', [epages.widget.FormBase], {
	
	// CREATE
		postCreate: function() {
			this.inherited('postCreate', arguments);
			
			this._handles.push([ this.domNode 		, 'change' 		, this._change ]);
			if( epages.Browser.engine=='MSIE' ) {
				this._handles.push([ this.domNode 		, 'click' 		, this._ieChangeFix ]);
			}
			
			
			this.attr( 'checked', this._attr.checked ? true : false );
			this.postAfterCreate();
		},
		
	// METHODS	
		// set/get formElement checked status
		_attrChecked: function(bool) {
			if(bool===true) {
				if( this._attr.type=='radio' ) {
					var radios = this._getRadios();
					for( var i=0,l=radios.length ; i<l ; i++ ) {
						radios[i].attr( 'checked', false );
						dojo.removeClass(radios[i].domNode,'DialogError');
						dojo.removeClass(radios[i].wrapNode,'DialogError');
					}
				}
				dj.attr( this.domNode, 'checked', true );
				dj.addClass( this.wrapNode, 'Checked' );
				return bool;
			}
			else if(bool===false) {
				dj.attr( this.domNode, 'checked', false );
				dj.removeClass( this.wrapNode, 'Checked' );
				return bool;
			}
			else {
				return this._attr.checked;
			}
		},
		// a fix for the IE to fire the change action instantly
		_ieChangeFix: function() {
			this.domNode.blur();
		},
		// handle the change
		_change: function() {
			this.attr( 'checked', dj.attr( this.domNode , 'checked' ) ? true : false );
			this.validate();
		},
		// found all radio element with same name in this form and outside of this form which added to this form by the form attribute
		_getRadios: function() {
			var domNodeName = dj.attr(this.domNode,'name');
			var formNodeName = dj.attr(this.formWdgt.domNode,'name');
			
			var inForm = dj.query('input[type=radio][name='+domNodeName+']',this.formWdgt.domNode);
			var exForm = dj.query('input[type=radio][name='+domNodeName+'][form='+formNodeName+']',document.body);

			var radios = inForm.concat(exForm);
			var required = this.attr('required');

			var ret = [];
			for( var i=0,l=radios.length ; i<l ; i++ ) {
				var wdgt =  $$( dj.attr(radios[i],'widgetid') );
				if( wdgt ) {
					ret.push( wdgt );
					required = wdgt.attr('required') || required;
					wdgt.attr('valide',true);
				}
			}
			this.attr('required',required);

			return ret;
		},
		
		_validate: function() {
			
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;	
			if( this._attr.type == 'checkbox' && ( (dj.attr(this.domNode,'checked') && this._attr.required) || !this._attr.required ) ) {
				valide = true;
			};
			
			if( this._attr.type == 'radio' ) {
				var radios = this._getRadios();
			}
			
			if( this._attr.type == 'radio' && ( (this.attr('checked') && this.attr('required')) || !this.attr('required') ) ) {
				valide = true;
			}
			else if( this._attr.type == 'radio' ) {
				for( var i=0,l=radios.length ; i<l ; i++ ) {
					valide = radios[i].attr('checked') || valide;
				}
			};
			return valide;
		}
	
	});
	
	
// WIDGET FormNumber
	dj.declare('epages.widget.FormNumber', [epages.widget.FormBase], {
		
		stepUpNode: 		null,			// node of the step up button
		stepDownNode: 		null,			// node of the step down button
		
		_stepCount:			null,			// counter of intervals
		_stepSpeed: 		null,			// interval time
		_stepInterval: 		null,			// interval of step action
		
	// CREATE
		postCreate: function() {
			this.inherited('postCreate', arguments);
			
			this._replacePattern();
			
		
			this.stepUpNode = dj.create('a',{
					'href':'javascript:;',
					'class':'StepUp'
				});
			
			this.stepDownNode = dj.create('a',{
					'href':'javascript:;',
					'class':'StepDown'
				});

			this.wrapNode.appendChild(this.stepUpNode);
			this.wrapNode.appendChild(this.stepDownNode);
			
			this._handles.push([ this.domNode 		, 'keyup' 		, this.validate ]);
			this._handles.push([ this.domNode 		, 'blur' 		, this.validate ]);
			
			this._handles.push([ this.stepUpNode 	, 'mousedown' 	, function(){ this._stepStart(+1) } ]);
			this._handles.push([ this.stepUpNode 	, 'mouseup'   	, this._stepStop ]);
			this._handles.push([ this.stepUpNode 	, 'mouseout'  	, this._stepStop ]);
			
			this._handles.push([ this.stepDownNode 	, 'mousedown' 	, function(){  this._stepStart(-1) } ]);
			this._handles.push([ this.stepDownNode 	, 'mouseup' 	, this._stepStop ]);
			this._handles.push([ this.stepDownNode 	, 'mouseout' 	, this._stepStop ]);
			
			// set pattern 
			this._attr.pattern = this._attr.pattern || epages.localize.getPattern( 'number' );
			
			this.postAfterCreate();
		},
		
	// METHODS		
		// handle the step 
		_stepAction: function( inc ) {
			this._stepCount++;
			this._stepSpeedHandle( inc );
			
			var value = dj.attr( this.domNode , 'value' );
			var ret;
			var step = inc * (this._attr.step || 1);
			var attrMin = this._attr.min;
			var attrMax = this._attr.max;			
				
				
			ret = epages.localize.str2number(value);
			ret = ret!==null ? ret : 0;
			var i = (ret+'').indexOf('.');
			var l = i>=0 ? (ret+'').substr(i+1).length : 0;
			ret = ret+step;
			ret = (attrMin && attrMin > ret.toFixed(l)) ? attrMin : (attrMax && attrMax < ret.toFixed(l)) ? attrMax : ret.toFixed(l);
			ret = epages.localize.number2str( ret );
			dj.attr( this.domNode , 'value' , ret );
		},
		// start the step action				
		_stepStart: function( inc ) {
			this._stepCount = 0;
			this._stepSpeed = 250;
			
			this._stepAction(inc);
			this._stepInterval = window.setInterval( dj.hitch(this,function(){ this._stepAction(inc) }) , this._stepSpeed );
		},
		// stop the step action				
		_stepStop: function() {
			window.clearInterval( this._stepInterval );
			epages.event.fire(this.domNode,'blur');
		},
		// handle the speed of the step action				
		_stepSpeedHandle: function( inc ) {
			if( dj.indexOf([10,30,50],this._stepCount)>=0 ) {
				this._stepSpeed = this._stepSpeed / 2;
				window.clearInterval( this._stepInterval );
				this._stepInterval = window.setInterval( dj.hitch(this,function(){ this._stepAction(inc) }) , this._stepSpeed );
			};
		},
		
		_validate: function() {
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;	
			
			if( value=='' && !this._attr.required ) {
				valide = true;
			}
			else if( value.match(new RegExp('^'+this._attr.pattern+'$')) ) {
				var val = epages.localize.str2number(value);
				var attrMin = this._attr.min;
				var attrMax = this._attr.max;
				valide = (attrMin && attrMin > val) ? false : (attrMax && attrMax < val) ? false : true;
				
			};
			return valide;
		}
	
	});
	
	
// WIDGET FormDate
	dj.declare('epages.widget.FormDate', [epages.widget.FormNumber], {
	
	// CREATE
		postCreate: function() {
			this.inherited('postCreate', arguments);
			
			// set pattern by type
			this._attr.pattern = epages.localize.getPattern( this._attr.type );
		},
		
	// METHODS					
		_stepAction: function( inc ) {
			this._stepCount++;
			this._stepSpeedHandle( inc );
			
			var value = dj.attr( this.domNode , 'value' );
			var ret;
			var attrStep = this._attr.step || 1;
			var attrMin = this._attr.min;
			var attrMax = this._attr.max;	
			
			var dateObj = epages.localize.date2obj( value , this._attr.type ) || new Date();
			switch( this._attr.type ) {
				case 'date':
					dateObj.setDate( dateObj.getDate() + (inc * attrStep) );
				break;
				case 'datetime':
					dateObj.setHours( dateObj.getHours() + (inc * attrStep) );
				break;
				case 'month':
					dateObj.setMonth( dateObj.getMonth() + (inc * attrStep) );
				break;
				case 'time':
					dateObj.setHours( dateObj.getHours() + (inc * attrStep) );
				break;
				case 'week':
					dateObj.setDate( dateObj.getDate() + (inc * attrStep * 7) );
				break;
			};
			var dateTime = dateObj.getTime()/1000;
				dateTime = (attrMin && attrMin > dateTime) ? attrMin : (attrMax && attrMax < dateTime) ? attrMax : dateTime;
				
			dateObj.setTime( dateTime*1000 );
			
			ret = epages.localize.obj2date( dateObj , this._attr.type );
			
			dj.attr( this.domNode , 'value' , ret );
		},
		
		_validate: function() {
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;	
			
			if( value=='' && !this._attr.required ) {
				valide = true;
			}
			else if( value.match(new RegExp('^'+this._attr.pattern+'$')) ) {
				var dateTime = epages.localize.date2obj( value , this._attr.type ).getTime()/1000;
				var attrMin = this._attr.min;
				var attrMax = this._attr.max;
				
				valide = (attrMin && attrMin > dateTime) ? false : (attrMax && attrMax < dateTime) ? false : true;
				
			};
			return valide;
		}
	
	});

})(dojo);