<?php
# php generate.php
# -- Runs everything in the modules directory
# -- use "--quiet" to prevent creating a warnings file

if (array_key_exists('HTTP_HOST',$_SERVER)) {
  die('Run from command line');
}
$STARTZEIT = time();

ini_set('memory_limit', '1024M');
ini_set('display_errors', 1);
error_reporting(E_ALL ^ E_NOTICE);
$debug = true;
require_once ('lib/parser/OutputControler.inc');
require_once('lib/parser/dojo.inc');
require_once('lib/parser/warnings.inc');   
// Use file serializers by default

$keys = array();
$namespaces = array();
$args = array();
$kwargs = array();
foreach (array_slice($argv, 1) as $arg) {
  if ($arg{0} == '-') {
    if (preg_match('%^--(quiet)=([^ ]+)$%', $arg, $match)) {
      $args[$match[1]] = $match[2];
    }
  }
}

// Do this in 3 parts
// 1. Turn all variables into single objects
// 2. Internalize class members
// 3. Serialize objects
$settings_ini = parse_ini_file("config.ini",true);
$quiet = is_string($args['quiet']) ? $args['quiet'] : (int)$settings_ini['DocSettings']['quiet'];
$GLOBALS['settings_ini'] = &$settings_ini;
$GLOBALS['print'] = new OutputControler($quiet);
$GLOBALS["warning"]->quiet = $quiet;
$GLOBALS['warning']->setEneabledWarnings($settings_ini['EnabledWarnings']);

if (array_key_exists('outdir',$settings_ini['DocSettings'])) {
	$kwargs['outdir'] = $settings_ini['DocSettings']['outdir'];
}

require_once('lib/generator/file/Freezer.php');
require_once('lib/generator/file/Serializer.php');
require_once('lib/generator/JsonSerializer.php');
require_once('lib/generator/JsonStoreSerializer.php');
require_once('lib/generator/HtmlSerializer.php');



if (array_key_exists('outdir',$kwargs)) {
   JsonSerializer::clean($kwargs['outdir']);
}
else {
   JsonSerializer::clean('cache');
}
//Freezer::clean('cache','nodes');
$files = dojo_get_files();
$nodes = new Freezer('cache', 'nodes');

$GLOBALS['print']->out("=== PARSING FILES ===\n",0);
$GLOBALS['print']->out("|                                                  |\n|",0);

flush();
$roots = array();
$percent = 0;
$percentDec = 0;
foreach ($files as $pos=>$set){
  list($namespace, $file) = $set;
  if (!array_key_exists($namespace, $namespaces)) {
    $namespaces[$namespace] = true;
  }       
  $new_percent = floor($pos / count($files) * 100);
  if (intval($new_percent / 2) > $percentDec) {
    $diff = intval($new_percent / 2)-$percentDec;
    for($i=0;$i<$diff;$i++)
        $GLOBALS['print']->out("#",0);
    $percentDec = intval($new_percent / 2);
  }
  $percent = $new_percent;
  $GLOBALS["warning"]->currentFile = $namespace . '/' . $file;
  flush();

  $contents = dojo_get_contents($namespace, $file);

  $provides = $contents['#provides'];
  unset($contents['#provides']);
  $resource = $contents['#resource'];
  unset($contents['#resource']);
  $requires = $contents['#requires'];
  unset($contents['#requires']);

  foreach ($contents as $var => $content) {
    foreach ($content as $key_key => $key_value) {
      $key_type = 'undefined';
      if (is_numeric($key_value)) {
        $key_type = 'numeric';
      }elseif(is_array($key_value)) {
        $key_type = 'array';
      }elseif(is_bool($key_value)) {
        $key_type = 'bool';
      }elseif(is_string($key_value)) {
        $key_type = 'string';
      }
      $keys[$key_key][] = $key_type;
      $keys[$key_key] = array_unique($keys[$key_key]);
    }
    $node = $nodes->open($var, array());

    if (trim($content['type']) && (empty($node['type']) || $content['type'] != 'Object')) {
      $node['type'] = $content['type'];
    }
    
    // Handle file-level information
    if ($provides && (!is_array($node['#provides']) || !in_array($provides, $node['#provides']))) {
      $node['#provides'][] = $provides;
    }

    if (!is_array($node['#namespaces']) || !in_array($namespace, $node['#namespaces'])) {
      $node['#namespaces'][] = $namespace;
    }

    $node['#resource'][] = "$namespace/$resource";
    $node['#requires'] = $requires;     
    if (!empty($content['tags'])) {
      $node['tags'] = $content['tags'];
    }

    if (!empty($content['private'])) {
      $node['private'] = $content['private'];
    }

    if (!empty($content['private_parent'])) {
      $node['private_parent'] = $content['private_parent'];
    }

    if (trim($content['summary'])) {
      $node['summary'] = $content['summary'];
    }

    if (trim($content['description'])) {
      $node['description'] = $content['description'];
    }

    if (trim($content['exceptions'])) {
      $node['exceptions'] = $content['exceptions'];
    }

    if ($content['private']) {
      $node['private'] = $content['private'];
    }
    if($content['protected']){
      $node['protected'] = $content['protected'];
    }
    if ($content['initialValue']) {
      $node['initialValue'] = $content['initialValue'];
    }
    if ($content['private_parent']) {
      $node['private_parent'] = $content['private_parent'];
    }
    if ($content['attachPoints']) {
      $node['attachPoints'] = $content['attachPoints'];
    }  
    if ($content['#LineNumber']) {
      $node['lineNumber'] = $content['#LineNumber'];
    }  
    if ($content['#SourceLocation']) {
      $node['sourceLocation'] = $content['#SourceLocation'];
    }  
    if (is_array($content['alias'])) {
      foreach ($content['alias'] as $alias) {
        $node['alias'] = $alias;
      }
    }

    if (is_array($content['examples'])) {
      foreach ($content['examples'] as $example) {
        if (!is_array($node['examples']) || !in_array($example, $node['examples'])) {
          $node['examples'][] = $example;
        }
      }
    }

    if ($content['instance']) {
      $node['instance'] = $content['instance'];
    }
    
    if ($content['prototype']) {
      $node['prototype'] = $content['prototype'];
    }
    if (!is_array($node['returns'])) {
      $node['returns'] = array();
    }
    if (trim($content['returns'])) {
      $node['returns'] = array_unique(array_merge($node['returns'], explode('|', $content['returns'])));
    }

    if (trim($content['return_summary'])) {
      $node['return_summary'] = $content['return_summary'];
    }

    foreach (array('prototype', 'instance', 'normal') as $scope) {
      if (!empty($content['mixins'][$scope])) {
        if (empty($node['mixins'][$scope])) {
          $node['mixins'][$scope] = array();
        }
        $node['mixins'][$scope] = array_unique(array_merge($node['mixins'][$scope], $content['mixins'][$scope]));
      }
    }

    if ($content['type'] == 'Function') {
      if ($content['classlike']) {
        $node['classlike'] = true;
      }

      if ($node['chains']) {
        if (!$content['chains']['prototype']) {
          $content['chains']['prototype'] = array();
        }
        $node['chains']['prototype'] = array_unique(array_merge($node['chains']['prototype'], $content['chains']['prototype']));
        if (!$content['chains']['call']) {
          $content['chains']['call'] = array();
        }
        $node['chains']['call'] = array_unique(array_merge($node['chains']['call'], $content['chains']['call']));
      }
      else {
        $node['chains']['prototype'] = ($content['chains']['prototype']) ? $content['chains']['prototype'] : array();
        $node['chains']['call'] = ($content['chains']['call']) ? $content['chains']['call'] : array();
      }

      if ($content['chains']) {
        unset($content['chains']['prototype']);
        unset($content['chains']['call']);
        $types = array_keys($content['chains']);
        if (!empty($types)) {
          print_r($types);
          die('Unexpected chain type');
        }
      }

      if (!empty($content['parameters'])) {
        if (!empty($node['parameters'])) {
          $node_parameters = array_keys($node['parameters']);
          $content_parameters = array_keys($content['parameters']);
          $long_parameters = (count($node_parameters) > count($content_parameters)) ? $node_parameters : $content_parameters;
          $short_parameters = (count($node_parameters) >  count($content_parameters)) ? $content_parameters : $node_parameters;

          $match = true;
          foreach ($long_parameters as $i => $parameter) {
            if ($i < count($short_parameters) && $parameter != $short_parameters[$i]) {
              $match = false;
            }
          }

          if ($match) {
            // Only process these if they match the first occurence
            foreach ($content['parameters'] as $parameter_name => $parameter) {
              if (empty($node['parameters'][$parameter_name]['type'])) {
                $node['parameters'][$parameter_name]['type'] = $parameter['type'];
              }
              if (trim($parameter['summary'])) {
                $node['parameters'][$parameter_name]['summary'] = $parameter['summary'];
              }
            }
          }
        }
        else {
          $node['parameters'] = $content['parameters'];
        }
      }
    }
    
    $parts = explode('.', $var);
    if (count($parts) > 1) {    
        $childID = array_pop($parts);      
        $parent = implode('.', $parts); 
        $parentNode = $nodes->open($parent, array());
        
        if (!array_key_exists($parent, $roots)) {
          $roots[$parent] = array(); 
        }
        if (!array_key_exists($var, $roots) && $node['classlike']) {
          $roots[$var] = array(); 
        }
        if (!array_key_exists($var, $roots)) {
          $parentNode['#children'][$childID] = $node;
        }
        $nodes->save($parent, $parentNode);
    }
    $nodes->save($var, $node);
  }
}
$GLOBALS["warning"]->currentFile = NULL;
unset($resources);

flush();

$GLOBALS['print']->out("#|\n=== SERIALIZING OBJECTS ===\n",0);
$GLOBALS['print']->out("|                                                  |\n|",0);

// Aggregate and save
$serializers = array();
if (!$kwargs['outdir']) {
	$kwargs['outdir'] = 'cache';
}

if($settings_ini['DocSettings']['outtype']=="json"){
    $serializers['json'] = new JsonSerializer($kwargs['outdir'], 'json');
} else if($settings_ini['DocSettings']['outtype']=="html"){
    $serializers['html'] = new HtmlSerializer($kwargs['outdir'], 'html');
    $serializers['html']->setTemplate('template.html');
    $storeSerializer = new JsonStoreSerializer($kwargs['outdir'], 'json');
}
if(!count($serializers)){
  die('Select a proper output type in config.ini > DocSettings > outtype');
}
$namespaceProvides = array();
$currentID = 0;
$percentDec = 0;
$percent = 0;
$pos = 0;
foreach ($roots as $id => $root) {
  $new_percent = floor($pos++ / count($roots) * 100);
  if (intval($new_percent / 2) > $percentDec) {
    $diff = intval($new_percent / 2)-$percentDec;
    for($i=0;$i<$diff;$i++)
        $GLOBALS['print']->out("#",0);
    $percentDec = intval($new_percent / 2);
  }
  $percent = $new_percent;
  
  $parts = explode('.', $id);
  if(!$id || !count($parts) || ($parts[0]!="epages" && $parts[0]!="dijit" && $parts[0]!="dojo" && $parts[0]!="dojox")){
    // Minor bug
    continue;
  }
  if(!$namespaceProvides[$parts[0]]){
      $namespaceProvides[$parts[0]] = new Freezer();
      $namespaceNode = $nodes->open($parts[0], array());
      $provides = $namespaceNode["#provides"];
      if($provides){
	      foreach($provides as $prov){
	          $namespaceProvides[$parts[0]]->save(strtolower($prov));
	      }
      }
  }
  if(!$namespaceProvides[$parts[0]]->isElement(strtolower($id))){
     // continue;
  }
  $node = $nodes->open($id, null);
  getChildren($id, $node, $nodes,$roots);
  flush();

  foreach ($serializers as $serializer) {
    $serializer->setObject($id, $node);
  }
  if($settings_ini['DocSettings']['outtype']=="html"){
	  if(count($parts)==1){
	  	//we found epages, dojo, dijit
	  	$provides = $node["#provides"];
	  	if($GLOBALS["packageDeclares"][$parts[0]]){
	  		$provides = array_unique(array_merge($provides,$GLOBALS["packageDeclares"][$parts[0]]));
	  	}
	  	if($provides){
	  		sort($provides);
	  	}
	  	$wood = array();
	  	$index = 0;
	  	if($provides){
		  	foreach($provides as $provide){
		  		$nameparts = explode(".",$provide);
		  		$cur = &$wood;
		  		if($nameparts[0] == $parts[0]){
			  		foreach($nameparts as $name){
			  			if(!$cur[$name]){
			  				$cur[$name] = array();
			  				$cur[$name]["#INDEX"] = $index++;
			  			}
			  			$cur = &$cur[$name];
			  		}
		  		}
		  	}
		  	$storedata = array();
		  	foreach($wood as $treeName => $content){
		  		$storedata[] = ConvertToStoreFormat("",$treeName,$content);
		  	}
		  	$storeSerializer->set($id,$storedata);
	  	}
	  }
  }
}
// try something new
//get back to the old stuff
$GLOBALS["warning"]->consoleReport();
$GLOBALS['warning']->generateReport(); 

$GLOBALS['print']->out("\nLaufzeit: ".(time() - $STARTZEIT)."\n",0);      

// * Assemble parent/child relationships
?>