<?php

class HtmlSerializer extends Serializer
{
  protected $header = array('');
  protected $footer = array('');
  protected $htmlTemplate = NULL;
  protected $extension = "html";

  public function setTemplate($filename){
      $this->htmlTemplate = $filename;
  }
  public function toObject($raw, $id=null) {
    return $raw;
  }
  private function getTemplate($obj){
    ob_start();
    require $this->htmlTemplate;
    $out = ob_get_clean();
    return $out;
  }
  private function ensureAvailability(&$obj,$vars){
      foreach($vars as $var){
          if(!$obj[$var]){
              $obj[$var] = array();
          }
      }
  }
  public function toString($raw, $id=null) {
    if (!$id) {
      if (!($id = $raw['id'])) {
        throw new Exception('toString must be passed an ID or raw object must contain and ID');
      }
    }
    $this->ensureAvailability($raw,array('superclass','methods','properties','provides'));
    return $this->getTemplate($raw);
  }

  protected function descend($node, $object) {
    if (!is_array($object) && !is_object($object)) {
      return $object;
    }

    foreach ($object as $key => $value) {
      switch ($key{0}) {
      case '@':
        $node[substr($key, 1)] = $value;
        break;
      case '#':
        if ($key == '#mixins') {
          foreach ($value as $mixins) {
            $scope = $mixins['@scope'];
            foreach ($mixins['#mixin'] as $mixin) {
              $node['mixins'][$scope][] = $this->descend(array(), $mixin);
            }
          }
        }
        elseif (count($value) == 1 && !in_array($key, array('#property', '#method', '#resource', '#require', '#parameter','#example', '#provide', '#return-type'))) {
          $node[substr($key, 1)] = $this->descend(array(), $value[0]);
        }
        else {
          foreach ($value as $child) {
            $node[] = $this->descend(array(), $child);
          }
        }
        break;
      default:
        if ($key === 'content') {
          if (count($object) == 1) {
            return $value;
          }
          $node['content'] = $value;
        }
      }
    }
    
    return $node;
  }

  public function convertToRaw($object) {
    return $this->descend(array(), $object);
  }
}

?>