<?php

require_once('JavaScriptStatements.php');
require_once('JavaScriptArray.php');
require_once('Destructable.php');

class JavaScriptFunctionCall extends Destructable {
  protected $call;
  protected $args;

  protected $assignment;
  protected $resolved_args;
  public $line_number;
  protected $name;
  protected $global_scope;

  public function __construct($call, $args, $line_number,$instantiated = FALSE) {
    $this->call = $call;
    $this->args = $args;
    $this->line_number = $line_number;
  }

  public function __destruct() {
    $this->mem_flush('call', 'args', 'assignment', 'resolved_args', 'name', 'global_scope');
  }
  public function getLine(){
      if($this->call){
        return $this->call->line;
      } else {
          return "";
      }
  }
  public function getLineNumber(){
      if($this->call){
        return $this->call->line_number;
      } else {
          return "";
      }
  }
  public function getCall(){
  	return $this->call;
  }
  private function resolve() {
    if (!$this->call->is_lookup()) {
      $this->global_scope = FALSE;
      $this->name = NULL;
    }
    else {
      list ($this->global_scope, $this->name) = $this->call->resolve();
    }
  }

  public function name() {
    if (!isset($this->name)) {
      $this->resolve();
    }
    return $this->name;
  }

  public function setAssignment($assignment) {
    $this->assignment = $assignment;
  }

  public function assignment() {
    if ($this->assignment) {
      if ($this->assignment->is_lookup()) {
        list(, $assignment) = $this->assignment->resolve();
        return $assignment;
      }
    }
    return NULL;
  }

  public function is_global () {
    if (!isset($this->global_scope)) {
      $this->resolve();
    }
    return !!$this->global_scope;
  }

  public function type() {
    // TODO: Try to resolve return type
    return 'Object';
  }

  public function arguments() {
    if (isset($this->resolved_args)) {
      return $this->resolved_args;
    }
    return ($this->resolved_args = new JavaScriptArray($this->args,$this->line_number));
  }
  public function convertToString(){
      if($this->name()){
          $result = $this->name();
          $result .="(";
          if($this->args){
            $result .="(".$this->arguments()->convertToString(TRUE).")";
          }
          $result .=")";
          return $result;
      }
      return "";
  }
}