window.showSource = function (/*String*/ filePath,/*int*/ lineNumber,onFinish){
	toggleSourcePane(true);
	if (window.currentFile != filePath) {
		window.styledSource.innerHTML = "Loading...";
		window.currentFile = filePath;
		if (!window.cachedSourceFiles[filePath]) {
			loadFile(filePath, lineNumber,onFinish);
			return;
		} else {
			setSourceContent(window.cachedSourceFiles[filePath],window.styledSource);
		}
	}
	scrollToSourceLine(lineNumber,window.styledSource);
	if(onFinish){
		onFinish(lineNumber,window.styledSource);
	}
};
window.toggleSourcePane = function (/*Boolean*/ state){
	var apiPane = dijit.byId("apiBorderPane");
	if(apiPane){
		if(state){
			window.styledSource.style.display="";
			apiPane.layout();
		} else {
			window.styledSource.style.display="none";
			apiPane.layout();
		}
	}
};
window.scrollToSourceLine = function (/*int*/ lineNumber,/*DomNode*/ sourceNode) {
	lineNumber++;
	var target = dojo.query('div.number'+lineNumber, sourceNode);
	if(target.length){
		smoothScroll( {
			node : target[0],
			win : sourceNode,
			duration : 600
		}).play();
	}
};
window.setSourceContent = function (/*String*/ text,node){
	node.innerHTML = "";
	text.replace(/'/g,"\'");
	var scr = dojo.create("script",{
		innerHTML : '<![CDATA['+text+']]>',
		type : "syntaxhighlighter",
		'class' : "brush: js"
	},node);
	SyntaxHighlighter.highlight();
	node.firstChild.style.display = "table-cell";
};
window.loadFile = function (/*String*/ file,/*int*/ lineNumber,onFinish) {
	dojo.xhrGet({
		url: "../../" + file,
		type: "text",
		sync: false,
		load: function(text)  {
			window.cachedSourceFiles[file] = text;
			setSourceContent(text,window.styledSource);
			scrollToSourceLine(lineNumber,window.styledSource);
			if(onFinish){
				onFinish(lineNumber,window.styledSource);
			}
		}
	});
};
window.togglePrivate = function (/*Boolean*/ state, /*DomNode*/ domNode){
	toggleVisbility(state,"private",domNode);
};

window.toggleProtected = function (/*Boolean*/ state,/*DomNode*/ domNode){
	toggleVisbility(state,"protected",domNode);
};
window.togglePublic = function (/*Boolean*/ state,/*DomNode*/ domNode){
	toggleVisbility(state,"public",domNode);
};
window.toggleInherited = function (/*Boolean*/ state,/*DomNode*/ domNode){
	toggleVisbility(state,"inherited",domNode);
};
window.toggleVisbility = function (/*Boolean*/ state,/*String*/ classname,/*DomNode*/ domNode){
	var queryNodes = dojo.query("."+classname,domNode);
	if(queryNodes.length){
		if(state){
			queryNodes.attr("style","display:;");
		} else {
			queryNodes.attr("style","display:none;");
		}
	}
};
window.smoothScroll = function (args) {
	// NB: this is basically dojox.fx.smoothScroll, but for some reason
	// smoothScroll uses target.x/y instead
	// of left/top. dojo.coords is returning a different y than the top for some
	// reason. Maybe position will
	// be better post 1.3
	if (!args.target) {
		args.target = dojo.coords(args.node, true);;
		if(args.node.nodeName == "TR" && args.node.offsetParent){
			args.target.t += args.node.offsetParent.offsetTop-3;
			args.target.l += args.node.offsetParent.offsetLeft;
		}
	}
	var _anim = function(val) {
		args.win.scrollLeft = val[0];
		args.win.scrollTop = val[1];
	};

	var anim = new dojo._Animation(dojo.mixin( {
		beforeBegin : function() {
			if (this.curve) {
				delete this.curve;
			}
			var current = {
				x : args.win.scrollLeft,
				y : args.win.scrollTop
			};
			anim.curve = new dojox.fx._Line( 
					[ current.x, current.y ],
					[args.target.l, args.target.t]
			);
		},
		onAnimate : _anim
	}, args));
	return anim;
};
window.expandNode = function (/*String*/ id,/*String*/ hrefNode,/*String*/ ext){
	var node = dojo.byId(id);
	if(!node){
		return;
	}
	if(!ext){
		ext = '';
	}
	if(hrefNode){
		var imgNode = dojo.query("img",hrefNode);
	}
	if(node.style.display == "none"){
		dojo.fx.wipeIn({
			node: node,
			duration: 100
		}).play();
		if(imgNode && imgNode.length){
			imgNode[0].src = "images/navigate_minus"+ext+".png";
		}
	} else {
		dojo.fx.wipeOut({
			node: node,
			duration: 100
		}).play();
		if(imgNode && imgNode.length){
			imgNode[0].src = "images/navigate_plus"+ext+".png";
		}
	}
};
window.loadContent = function (/*String*/ id,/*Boolean?*/ scrollToElement){
	if(scrollToElement){
		scrollToElement = id.substr(id.lastIndexOf(".")+1);
		id = id.substr(0,id.lastIndexOf("."));
	}
	id = id.replace(/\./g,"/");
	id += ".html";
	id = id.toLowerCase();
	toggleSourcePane(false);
	dojo.xhrGet({
		url: "documentation/"+id,
		sync: true,
		handleAs: "text",
		load: function(result){
			var apiPane = dijit.byId("apiPane");
			var cP = apiPane.addContent(id,result);
			togglePrivate(false,cP);
			toggleInherited(false,cP);
			SyntaxHighlighter.highlight();
			if(scrollToElement){
				var target = dojo.query('tr[name="'
						+ scrollTo
						+ '"]', cP);
				if (target.length) {
					var node = target[0];
					node.style.display = "";
					var ext = dojo.query(".toggleLink",node);
					for(var i=0,l=ext.length;i<l;i++){
						ext[i].onclick();
					}
					var anim = smoothScroll( {
						node : node,
						win : cP,
						duration : 600
					}).play();
				}
			}
		},
		error: function(){console.warn(arguments);}
	});
};
window.filterTree = function (/*String*/ key){
	var filterString = searchInput.attr("value");
	
	window.loadedPackages[window.currentPackage].applyQuery({
		label: "*"+filterString+"*"
	});
	window.loadedPackages[window.currentPackage].showingAll = false;
	if(filterString === "" && !window.loadedPackages[window.currentPackage].showingAll){
		window.loadedPackages[window.currentPackage].showingAll = true;
		window.loadedPackages[window.currentPackage].applyQuery({
			label: "*"
		},{
			deep:false
		});
	}
};
window.loadPackage = function (packageName){
	window.currentPackage = packageName;
	if(!window.loadedPackages[packageName]){
		dojo.xhrGet({
			url: "documentation/"+packageName+".json",
			sync: false,
			handleAs: "json",
			error: function(){
				console.warn("Couldn't load molue "+packageName+" make shure you've run the generate.php")
			},
			load: function(result){
				var pK = dijit.byId("tabContainer");
				var store = new dojo.data.ItemFileReadStore({
					data: {
					identifier: 'id',
					label: 'label',
					items: result
				}
				});
				store.comparatorMap = {};
				store.comparatorMap["label"] = function(a,b) {
					var ret = 0;
					a = a.toLowerCase();
					b = b.toLowerCase();
					// We want to map these by what the priority of these items are, not by alphabetical.
					// So, custom comparator.
					if (a > b) {
						ret = 1;
					}
					if (a < b) {
						ret = -1;
					}
					return ret;
				};
	
				var treeModel = new epages.data.ForestStoreModel({
					store: store,
					queryOptions:{
					ignoreCase: true,
					deep: true
				},
				sort: [{
					attribute: "label"
				}],
				showingAll: true
				});
				var tree = new dijit.Tree({
					model: treeModel,
					showRoot: false,
					_createTreeNode: function(/*Object*/args) {
						var tnode = new dijit._TreeNode(args);
						tnode.labelNode.innerHTML = args.label;
						return tnode;
					},
					onClick : function(node){loadContent(node.fullstring[0]);}
				}, "treeNode:"+packageName);
				var node = tree.rootNode.getChildren();
				if(node && node.length){
					tree._expandNode(node[0]);
				}
				window.loadedPackages[packageName] = treeModel;
				pK.layout();
			},
			error: function(){console.warn(arguments);}
		});
	}
};