/*
	Copyright (c) 2006-2009, epages Software GmbH
	All Rights Reserved.
*/
dojo.provide('epages.cartridges.de_epages.externalcontent.twitter');
dojo.require('epages.cartridges.de_epages.design.mcecontent');
dojo.require('epages.io.text');

epages.cartridges.de_epages.externalcontent.Twitter = epages.Class();

dojo.extend(epages.cartridges.de_epages.externalcontent.Twitter,
	epages.cartridges.de_epages.design.MceContent.prototype, {
	contentClass         : 'epages.cartridges.de_epages.externalcontent.Twitter',
	title                : 'Twitter',
	// general
	width                : 250,
	height               : 300,
	username             : '',
	tweetInterval				 : 6000,
	numberOfTweets			 : 4,
	// features
	includeScrollbar     : false,
	loopResults					 : true,
	pollForNewResults    : true,
	showHashtags				 : true,
	showTimestamps			 : true,
	showAvatars					 : false,
	behavior						 : 'all',
	// theme
	shellBgColor         : '',
	shellTextColor       : '',
	tweetBgColor         : '',
	tweetTextColor       : '',
	tweetLinkColor       : '',
	// rest
	twitterscript        : 'http://widgets.twimg.com/j/2/widget.js',	// const
	directPreview        : false,
	preview              : epages.vars.StoreRoot + '/BO/icons/navelement_twitter.png',
	fototitle            : 'Microbloggin with Twitter',

	info: function () {
		// summary: Compose info objects from class members.
		// description: Extension to MceContent's info() method. All relevant additonal parameters
		//	from this Twitter class are added to the info object before returning it.
		var info = epages.cartridges.de_epages.design.MceContent.prototype.info.apply(this, arguments);
		dojo.mixin(info, {
			username             : this.username,
			width             	 : this.width,
			height             	 : this.height,
			shellBgColor         : this.shellBgColor,
			shellTextColor       : this.shellTextColor,
			tweetBgColor         : this.tweetBgColor,
			tweetTextColor       : this.tweetTextColor,
			tweetLinkColor       : this.tweetLinkColor,
			twitterscript        : this.twitterscript,
			tweetInterval				 : this.tweetInterval,
			numberOfTweets			 : this.numberOfTweets,
			includeScrollbar     : this.includeScrollbar,
			loopResults					 : this.loopResults,
			pollForNewResults    : this.pollForNewResults,
			showHashtags				 : this.showHashtags,
			showTimestamps			 : this.showTimestamps,
			showAvatars					 : this.showAvatars,
			behavior						 : this.behavior
		});
		return info;
	},

	saveForm: function() {
		// summary: Action when the setup form for the Twitter parameters is saved.
		// description: Retrieves the data from the template's formular fields into the
		//		this.formData member variable. Updates the state of the loopOldTweets
		//		flag from the form input value.
		if (this.form !== undefined){
			this.formData = (new epages.html.Form(this.form)).get();
		}
  },

	contentString: function (info, ispreview) {
		// summary: Method called by base class handler to return the content string of the gadget.
		// description: First retrieves the settings for the twitter gadget and then returns the
		//	composed html code string.
		var finalstring = '';

		if(info.formData.get('twitterUserName') && info.formData.get('twitterUserName').length > 0){
			info = this._getTwitterData(info.formData);
			if(info) {
				finalstring = this._getTwitterString(info);
			}
		}
		return finalstring;
	},

	loadPreview: function (domObject) {
		// summary: Action when clicked on the preview button.
		// description: Uses a small html snippet from the twitterpreview.html file to insert the
		//	twitter code snippet to the iframe body.
		if(this.formData.get('twitterUserName') && this.formData.get('twitterUserName').length > 0){
			var info = this._getTwitterData(this.formData);
			var url = 'twitterpreview.html?';

			for(var key in info) {
				url += key+'='+escape(info[key])+'&';
			}
			url = url.substr(0, url.length-1); // strip last "&"
			domObject.src = dojo.moduleUrl('epages.cartridges.de_epages.externalcontent',url);
		}
	},

	_getTwitterData: function (sourceobject) {
		// summary: Private method to fetch the gadget setting from the template's form inputs.
		// description: Composes and returns an info object containing the settings for username, sizes,
		//		colors, etc. of the twitter gadget.
		var info = {};
		info.username = 					sourceobject.get('twitterUserName')[0] != undefined ? sourceobject.get('twitterUserName')[0] : '';

		info.height =							sourceobject.get('twitterHeight')[0] != undefined ? parseInt(sourceobject.get('twitterHeight')[0]) : this.height;
		if(sourceobject.get('twitterAutoWidth')[0]) {
			info.width = 'auto';
		} else {
			info.width =						sourceobject.get('twitterWidth')[0] != undefined ? parseInt(sourceobject.get('twitterWidth')[0]) : this.width;
		}

		info.shellBgColor = 			sourceobject.get('twitterShellBgColor')[0] != undefined ? sourceobject.get('twitterShellBgColor')[0] : '';
		info.shellTextColor = 		sourceobject.get('twitterShellLinkColor')[0] != undefined ? sourceobject.get('twitterShellLinkColor')[0] : '';
		info.tweetBgColor = 			sourceobject.get('twitterTweetBgColor')[0] != undefined ? sourceobject.get('twitterTweetBgColor')[0] : '';
		info.tweetTextColor = 		sourceobject.get('twitterTweetTextColor')[0] != undefined ? sourceobject.get('twitterTweetTextColor')[0] : '';
		info.tweetLinkColor = 		sourceobject.get('twitterTweetLinkColor')[0] != undefined ? sourceobject.get('twitterTweetLinkColor')[0] : '';

		info.twitterscript =      this.twitterscript;

		info.tweetInterval =			sourceobject.get('twitterTweetInterval')[0] != undefined ? parseInt(sourceobject.get('twitterTweetInterval')[0])*1000 : this.tweetInterval;
		info.numberOfTweets =			sourceobject.get('twitterNumberOfTweets')[0] != undefined ? parseInt(sourceobject.get('twitterNumberOfTweets')[0]) : this.numberOfTweets;

		info.includeScrollbar =   sourceobject.get('twitterIncludeScrollbar')[0] ? true : false;
		info.loopResults =				sourceobject.get('twitterLoopResults')[0] ? true : false;
		info.pollForNewResults =  sourceobject.get('twitterPollForNewResults')[0] ? true : false;
		info.showHashtags	=				sourceobject.get('twitterShowHashtags')[0] ? true : false;
		info.showTimestamps =		  sourceobject.get('twitterShowTimestamps')[0] ? true : false;
		info.showAvatars =				sourceobject.get('twitterShowAvatars')[0] ? true : false;

		info.behavior = this.behavior;
		var behaviourNode = sourceobject.get('twitterBehavior');
		for( var i=0,l=behaviourNode.length ; i<l ; i++ ) {
			if(behaviourNode[i]) {
				if(behaviourNode[i] == "LoadAllTweets") {
					info.behavior = 'all';
				} else if(behaviourNode[i] == "TimedInterval") {
					info.behavior = 'default';
				}
			}
		}

		return info;
	},

	_getTwitterString: function (info) {
		// summary: Using the given info object, compose the html code string of the gadget.
		// description: Fills the variable parameters into the twitter gadget code. The actual
		//	gadget is fetched from widgets.twimg.com via a script tag. Then a simply object instance
		//		of that gadget is created, supplied with all the setup parameters.

	  var finalstring =
			'<script src="'+info.twitterscript+'"></script>\n'+
			'<script>\n'+
			'new TWTR.Widget({\n'+
			'  version: 2,\n'+
			'  type: \'profile\',\n'+
			'  rpp: '+info.numberOfTweets+',\n'+
			'  interval: '+info.tweetInterval+',\n'+
			'  width: '+(/^\d+$/.test(info.width) ? info.width : '\''+info.width+'\'')+',\n'+
			'  height: '+info.height+',\n'+
			'  theme: {\n'+
			'    shell: {\n'+
	    '      background: \''+info.shellBgColor+'\',\n'+
	    '      color: \''+info.shellTextColor+'\'\n'+
			'    },\n'+
			'    tweets: {\n'+
	    '      background: \''+info.tweetBgColor+'\',\n'+
	    '      color: \''+info.tweetTextColor+'\',\n'+
	    '      links: \''+info.tweetLinkColor+'\'\n'+
			'    }\n'+
			'  },\n'+
			'  features: {\n'+
			'    scrollbar: '+info.includeScrollbar+',\n'+
			'    loop: '+info.loopResults+',\n'+
			'    live: '+info.pollForNewResults+',\n'+
			'    hashtags: '+info.showHashtags+',\n'+
			'    timestamp: '+info.showTimestamps+',\n'+
			'    avatars: '+info.showAvatars+',\n'+
			'    behavior: \''+info.behavior+'\'\n'+
			'  }\n'+
			'}).render().setUser(\''+info.username+'\').start();\n'+
			'</script>\n';
    return finalstring;
	}

});