use strict;
use Test::Simple tests => 3;
use Getopt::Long;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw (ConnectTestSite DisconnectSite);
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadClassByAlias LoadObjectByPath ExistsObjectByPath );

my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();

    # create test classes
    ok( Init(), "initialization" );

    my $Folder = LoadObjectByPath( $ROOTPATH );

    my $Class = LoadClassByAlias('Object');
    my $Count1 = scalar @{$Class->objects};
    $Folder->insertChild( $Class, { Alias => 'Store999' } );
    my $Count2 = scalar @{$Class->objects};
    ok($Count1 + 1 == $Count2, 'EPG-5394');

    ok( Cleanup(), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    if( ExistsObjectByPath( $ROOTPATH ) ) {
        LoadObjectByPath( $ROOTPATH )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();

    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
