/*
	Copyright (c) 2006-2009, epages Software GmbH
	All Rights Reserved.
*/
dojo.provide("epages.cartridges.de_epages.order.widget.DocumentGenerator");
dojo.require("epages.widget.LocalizedWidget");
dojo.require("epages.widget.Modaldialog");
dojo.require('dojo.date.locale');
dojo.require('epages.uimessagehandler');

dojo.declare (
	"epages.cartridges.de_epages.order.widget.DocumentGenerator",
	[epages.widget.LocalizedWidget],
	{
		/**
		 * public properties
		 */
		orders    : undefined,      // array if order object ids
		url       : '?',            // url for the json action(s)
		objectId  : undefined,      // object id for the json action(s)

		/**
		 * private properties
		 */
		_dialogWidget 				: undefined,  // modaldialog widget instance
		_createMode   				: undefined,  // string for the creation mode ("Create" or "CreateAndPrint")
		_numOrderDocuments 		: undefined,	// the number of documents of the order, used to store the JSON return data

		/**
		 * widget properties
		 */
		templatePath      : dojo.moduleUrl('epages.cartridges.de_epages.order.widget', 'templates/DocumentGenerator.html'),
		translationName   : dojo.moduleUrl('epages.cartridges.de_epages.order.widget', 'templates/translation'),
		widgetsInTemplate : true,

		postCreate: function() {
			// summary: Creates new Modal dialog
			this.inherited("postCreate", arguments);
			this.createModalDialog();
		},

		_checkCreateButton: function(evt) {
			// summary: Enable or disable the create button of the modaldialog.
			// description: Deactivates the create button (found using its classname "InsertButton") on the
			//    modaldialog if no document type for creation is selected. Re-Activates the button if one
			//    is selected.
			//    Callback method to be used by dojo.connnect.
			// example:
			//    dojo.connect(this.DocGen_PackingSlip.domNode, "onclick", this, "_checkCreateButton");
			var createButton = epages.html.getElementsByClassName("InsertButton", this.dialogNode.domNode)[0];

			if(!this.DocGen_PackingSlip.domNode.checked && !this.DocGen_Invoice.domNode.checked ||
				(
				 !this.DocGen_IssueDate.value ||
				  dojo.hasClass(this.DocGen_IssueDate.textBoxNode, "DialogError") ||
				 !this.DocGen_DeliveryDate.value ||
				  dojo.hasClass(this.DocGen_DeliveryDate.textBoxNode, "DialogError")
				)
				 ){
					dojo.addClass(createButton, "Disabled");
					dojo.removeClass(createButton, "Active");
			} else {
					dojo.removeClass(createButton, "Disabled");
					dojo.addClass(createButton, "Active");
			}
		},

		_createDocuments: function () {
				// summary: Exec JSON request to generate packing slips and invoices for the given orders.
				// description: Reads the chosen options from the modaldialog's form inputs. Then calles
				//    the JSON request using the JSONGenerateDocuments change action. This will generate
				//    a new packing slip and invoice for all orders in the this.orders array.
				//    New documents will only be generated if none exists yet.
				// example:
				//    if(!me._createDocuments()) {
				//      return; //error
				//    }
				// returns: true if the method succeeded, else false.

				// collect form data into vars
				var createPackingSlip = this.DocGen_PackingSlip.domNode.checked;
				var createInvoice = this.DocGen_Invoice.domNode.checked;

				if(this.ModeCreateAndPrint.domNode.checked) {
					this._createMode = this.ModeCreateAndPrint.domNode.value;
				} else if(this.ModeCreate.domNode.checked) {
					this._createMode = this.ModeCreate.domNode.value;
				} else {
					return false;
				}

				var ReadyForShippingOn = this.DocGen_ReadyForShippingOn.domNode.checked;
				var DispatchedOn = this.DocGen_DispatchedOn.domNode.checked;
				var InvoicedOn = this.DocGen_InvoicedOn.domNode.checked;

				var issueDate = epages.localize.formatDateTime(this.DocGen_IssueDate.value, epages.vars.Locale.dateFormat);
				var deliveryDate = epages.localize.formatDateTime(this.DocGen_DeliveryDate.value, epages.vars.Locale.dateFormat);

				dojo.require("epages.io.json");
				var json = new epages.io.Json();
				var result = json.loadSync(this.url, {
					ObjectID                : this.objectId,
					ChangeAction            : 'JSONGenerateDocuments',
					Locale                  : epages.vars.Locale.alias,
					OrderList               : this.orders,
					createPackingSlip       : createPackingSlip,
					createInvoice           : createInvoice,
					createMode              : this._createMode,
					IssueDate               : issueDate,
					DeliveryDate            : deliveryDate,
					ReadyForShippingOn      : ReadyForShippingOn,
					DispatchedOn            : DispatchedOn,
					InvoicedOn              : InvoicedOn
				});
				if(result.error || !result.data) {
					return false;
				}
				this._numOrderDocuments = parseInt(result.data.OrderDocumentCount);

				// check for error
				return true;
		},

		_displayHintBox: function () {
			// summary: Show hint box on top of orders grid table.
			// description: Uses the anchor div named "DocumentGeneratorMessage" and replaces its content
			//    with a information message area indicating that the document generation is finished.
			//    If the _createMode is set to CreateAndPrint, an additional print view link is included.
			//    In parallel a new popup window is opened containing the print view for the latest
			//    packing slip and invoice of all given orders.
			// example:
			//    this._displayHintBox();

			// Print view hint box
			var generatorMessage ='<h3>'+this.translate('DocumentsCreated')+'</h3>';
			var printViewUrl;
			if(this._createMode && this._createMode == "CreateAndPrint") {
				printViewUrl = '?ViewAction=MBO-ViewPrintMultipleDocuments&amp;ObjectID='+this.objectId+'&amp;ChangeAction=BlockHistory&amp;PopUp=1&amp;Print=1';

				// action ids of all orders
				for( var i=0,l=this.orders.length ; i<l ; i++ ) {
					printViewUrl += '&amp;ListObjectID='+this.orders[i];
				}
				generatorMessage += '<i class="Sprite ico_s_printer" /></i><a href="'+printViewUrl;
				generatorMessage += '" onclick="openWindow(this.href, \'\', \'\'); return false;"> '+this.translate('PrintDocuments')+'</a>';
			} else {
				generatorMessage += '<br />';
			}
			$('DocumentGeneratorMessage').innerHTML=generatorMessage;
			$('DocumentGeneratorMessage').className="DialogMessage MessagePositiveResult";

			// open browser popup to print view
			if(this._createMode && this._createMode == "CreateAndPrint") {
				openWindow(printViewUrl.replace(/&amp;/g, "&"), '', '');
			}
		},

		createModalDialog: function () {
			// summary: Sets up a new modal dialog for the document generator using a template.
			// description: Creates a new modaldialog instance with a create and a cancel button and
			//    using the html template from the this.templatePath variable.
			//    In addition, the two inputs determining the kind of document to be generated are
			//    connected to the _checkCreateButton method.
			//    When hitting the create button of the modaldialog, the dialog is hidden and a busy-
			//    layer is displayed during the document generation called via _createDocuments().
			//    Afterwards the busy layer is hidden and the hintbox is displayed indicating the
			//    success of the process.
			// example:
			//    this.createModalDialog();
			dojo.require("epages.widget.Modaldialog");
			var me = this;
			this._dialogWidget = new epages.widget.Modaldialog({
				options:{
					title   : me.translate('CreateAndPrintDocuments'),
					width   : 400,
					height  : 600,
					content : me.dialogNode
				},
				buttons: [{
					label: '{Create}',
					cssClass: 'Active InsertButton',
					onclick: function() {
						// do nothing if the button is disabled
						if(!dojo.hasClass(this.domNode, "Active")) {
							return;
						}
						me._dialogWidget.hide();
						dojo.publish("uimessage/startBusy", ["DocumentGeneration", me.translate('WaitForGenerateDocuments')+'...']);
						if(!me._createDocuments()) {
							return; //error
						}
						dojo.publish("uimessage/stopBusy", ["DocumentGeneration"]);
						me._displayHintBox();
						dojo.publish("documentgenerator/ready", [me._numOrderDocuments]);
					}
				}, {
					label: '{Cancel}',
					onclick: function() {
						me._dialogWidget.hide();
					}
				}]
			});
			document.body.appendChild(this._dialogWidget.domNode);

			this.connect(this.DocGen_PackingSlip.domNode, "onclick", "_checkCreateButton");
			this.connect(this.DocGen_Invoice.domNode, "onclick", "_checkCreateButton");
			this.connect(this.DocGen_IssueDate, "onValueInput", "_checkCreateButton");
			this.connect(this.DocGen_DeliveryDate, "onValueInput", "_checkCreateButton");
		},

		openDialog: function(aSelectedOrders) {
			// summary: Updates the dialog form and displays the modaldialog.
			// description: After updating the heading with the selected order count and the date
			//    input fields using today's date, the modaldialog - which was former has to be
			//    created using createModalDialog() method - is displayed.
			// example:
			//    $$('DocumentGenerator').openDialog(aSelectedOrders);
			this.orders = aSelectedOrders;
			// update template dom contents: heading and date fields
			this.dialogHeading.innerHTML = this.translate('SelectedOrders') + ": " + this.orders.length;

			var pattern = epages.localize.getDojoDateTimePattern(epages.vars.Locale.dateFormat);
			var todayDate = dojo.date.locale.format(new Date(), {selector:'date', datePattern:pattern, timePattern: ''});

			this.DocGen_IssueDate.setValue(todayDate);
			this.DocGen_DeliveryDate.setValue(todayDate);

			this._dialogWidget.show();
		}

	});
