dojo.provide("epages.cartridges.de_epages.presentation.droplist");
dojo.require("epages.browser");
dojo.require("epages.html.element");

epages.cartridges.de_epages.presentation.Droplist = function(opt) {
// summary: droplist constructor / initialize droplist
	this._setOptions(opt);

	var doc = document;

	// set up ghost
	this.ghostNode=doc.createElement((this.options.ghostNodeName !== undefined) ? this.options.ghostNodeName : "div");
	this.ghostNode.className=(this.options.ghostCssClass !== undefined) ? this.options.ghostCssClass : "Ghost";
	this.isGhostInList = false;

	// set up empty ghost
	this.emptyNode=doc.createElement((this.options.ghostNodeName !== undefined) ? this.options.ghostNodeName : "div");
	this.emptyNode.className=(this.options.ghostCssClass !== undefined) ? this.options.ghostCssClass+" Empty" : "Ghost Empty";
	this.emptyNode.appendChild(doc.createTextNode(this.options.listEmptyString));

	// set up new ghost
	this.newNode=doc.createElement((this.options.ghostNodeName !== undefined) ? this.options.ghostNodeName : "div");
	this.newNode.className=(this.options.ghostCssClass !== undefined) ? this.options.ghostCssClass+" New" : "Ghost New";
	this.newNode.innerHTML='<div class="ClickContainer"><div class="Container">'+this.options.addNewItemString+'</div></div>';
	if(this.options.newItemOnClick !== undefined) {
		this.newNode.firstChild.onclick = dojo.hitch(this, function() {
			dojo.publish("interactionRestrictor/suspend", ["newGhostRestrictor", this.newNode]);
			setTimeout(dojo.hitch(this, function() {
				this.options.newItemOnClick();
				setTimeout(function(){
					dojo.publish("interactionRestrictor/permit", ["newGhostRestrictor"])}, 400);
			}), 10);
		});
	}

	// dnd handler
	this.dndHandlerId=this.options.dndHandlerId;
	// listContainer
	this.listContainer = this.options.listContainer;
	this.sensitiveArea = (this.options.sensitiveArea) ? this.options.sensitiveArea : this.options.listContainer;

	this.listId=this.listContainer.id;

	// clean up list
	this._cleanUpList();

	// show empty Node
	this._insertEmptyGhost();

	// show new Node
	if(this.options.addNewItemString != ""){
		this._insertNewGhost();
	}

	// mark existing wrappers
	if(this.options.wrapperNodeName){
		this._markWrapper();
	}

	this._appendListeners();
};

epages.cartridges.de_epages.presentation.Droplist.prototype = {
	listContainer  : undefined,			// domnode   contains all the draggable nodes
	sensitiveArea  : undefined,			// domnode   draggables can be dropped inside the coordinates of this node
	listId         : undefined,			// string    id of listContainer node
	options        : undefined,			// object    options for constructor (see below)
	ghostNode      : undefined,			// domnode   node for dropghost
	isGhostInList  : undefined,			// boolean   indicates if the ghost / current drop target exsits in the list
	_isGhostSizeSet: undefined,			// boolean   inidcates if the ghost needs to be reszied / renderd again
	emptyNode      : undefined,			// domnode   will be inserted into empty list (show the user some text that he can insert something here)
	newNode        : undefined,			// domnode   add new item button

	_setOptions: function(options) {
		// summary: mix in options
		this.options = {
			// defaults
			dndHandlerId    : undefined,	// string    id of the main dnd handler (Dragdrop instance) / subscribe listeners to that id
			listContainer   : undefined,	// domnode   contains all the draggable nodes
			orientation     : undefined,  // string    orientation of list ("UpDown" or "LeftRight")
			wrapperNodeName : undefined,  // domnode   wrap dnd node nodename eg. div, td
			listEmptyString : '',					// string    this string will be inserted as text into empty node
			addNewItemString: '',					// string    this string will be used as label text for the emptyNode
			newItemOnClick  : undefined		// function  will be applied as click function on the newNode
		};
		dojo.mixin(this.options, options);
	},

	_appendListeners: function() {
	// summary: subscribe listeners to Dragdrop instance
		dojo.subscribe('dragdrop'+this.dndHandlerId+'/move',  this, "_move");
		dojo.subscribe('dragdrop'+this.dndHandlerId+'/drop',  this, "_drop");
		dojo.subscribe('dragdrop'+this.dndHandlerId+'/start', this, "_startDragging");

		dojo.subscribe('droplist'+this.listId+'/change', this, "_listChange");
	},

	_isInsideSensitiveArea: function(/* object */ dndPos) {
	// summary: returns true if mouse coordinates are inside the sensitive area
		var containerElement = this.sensitiveArea;
		var listContPos = $E(containerElement).getAbsolutePosition();
		if(dndPos.mouseY >= listContPos.y && dndPos.mouseY <= listContPos.y + containerElement.clientHeight &&
			 dndPos.mouseX >= listContPos.x && dndPos.mouseX <= listContPos.x + containerElement.clientWidth) {
			return true;
		}else {
			return false;
		}
	},

	_canInsertBefore: function(/* domnode */ node, /* object */ dndPos) {
	// summary: returns true if an element can be inserted before
		var curPos = $E(node).getAbsolutePosition();
		if(this.options.orientation == "UpDown") {
			return (curPos.y > dndPos.mouseY);
		}else {
			return (curPos.x > dndPos.mouseX);
		}
	},

	_startDragging: function(opt) {
	// summary: event function dragrop/start - initaliaze dragging
		this._isGhostSizeSet = false;
		if(this._isInsideSensitiveArea(opt)) {
			opt.node.ep_droplistid = this.listId;
		}
	},

	_move: function(opt) {
	// summary: event function dragrop/move - show ghost
		if(this._isInsideSensitiveArea(opt)) {
			// position in list
			var node = this.listContainer.firstChild;
			while(node != null) {
				if(node.nodeType != 3 && opt.node !== node.firstChild && this._canInsertBefore(node,opt)) {
					break;
				}
				if(node.className == "Ghost New") {
					break;
				}
				node = node.nextSibling;
			}

			// set ghost
			if(!this._isGhostSizeSet) {
				this._setGhostSize(opt.node);
			}

			if(node != null) {
				this._insertGhostBefore(node);
			}else {
				this._insertGhostEol();
			}
		}else {
			this._removeGhost();
		}
	},

	_drop: function(opt) {
	// summary: event function dragrop/drop - finish dragging / publish drop in list event / insert node
		if(this.isGhostInList) {
			// find insertBefore node
			if(opt.element.onDropInList) {
				var insertBefore = null;
				if(this.options.wrapperNodeName) {
					if(this.ghostNode.nextSibling && this.ghostNode.nextSibling.firstChild && this.ghostNode.nextSibling.className != "Ghost New") {
						var insertBefore = this.ghostNode.nextSibling.firstChild;
						while(insertBefore != null && insertBefore.nodeType != 1) {
							insertBefore=insertBefore.nextSibling;
						}
						if(insertBefore.nodeType != 1)	{
							insertBefore = null;
							console.warn("Droplist._drop: no valid successor node found");
						}
					}
				} else {
					insertBefore = this.ghostNode.nextSibling;
				}

				// external handler
				opt.element.onDropInList({
					listObject: this,            // list object
					insertBefore: insertBefore,  // dom object or null
					oldWrapper: (opt.node.parentNode.ep_isWrapper) ? opt.node.parentNode : undefined,
					newWrapperNodeName: (this.options.wrapperNodeName) ? this.options.wrapperNodeName : undefined
				});
			} else {
				// standard handler
				this._insertNode(opt);
			}

			if(opt.node.ep_droplistid){
				dojo.publish('droplist'+opt.node.ep_droplistid+'/change', []);
			}

			//set reference to new list
			opt.node.ep_droplistid=this.listId;

			// remove ghost
			this._removeGhost();
			this._isGhostSizeSet=false;

			// remove empty Node
			this._removeEmptyGhost();
		}
	},

	_insertNode: function(/* object */ opt) {
	// summary: insert dragged node
		// save old wrapper
		var oldwrapper = (opt.node.parentNode.ep_isWrapper) ? opt.node.parentNode : undefined;

		// create new wrapper
		if(this.options.wrapperNodeName !== undefined) {
			var wrapperNode=this._createWrapper();
			this.listContainer.insertBefore(wrapperNode, this.ghostNode);
			wrapperNode.appendChild(opt.node);
		} else {
			this.listContainer.insertBefore(opt.node, this.ghostNode);
		}

		// remove old wrapper
		if(oldwrapper){
			this._removeWrapper(oldwrapper);
		}
	},

	_setGhostSize: function(/* domnode */ srcNode) {
	// summary: resize ghost node
		if(this.options.orientation == "UpDown") {
			this.ghostNode.style.width=this.listContainer.offsetWidth+"px";
			if(srcNode.clientHeight!=0 && srcNode.offsetHeight!="0"){
				this.ghostNode.style.height=srcNode.offsetHeight+"px";
			}else{
				this.ghostNode.style.height="27px";	// dirty fix for problem with dragelements whithout height (floated content)
			}
		} else {
			this.ghostNode.style.height=this.listContainer.offsetHeight+"px";
			this.ghostNode.style.width=srcNode.offsetWidth+"px";
		}
		this._isGhostSizeSet=true;
	},

	_insertGhostBefore: function(/* domnode */ node) {
	// summary: insert ghost node before a list element
		this.listContainer.insertBefore(this.ghostNode, node);
		this.isGhostInList = true;
		this._removeEmptyGhost();
	},

	_insertGhostEol: function() {
	// summary: insert ghost node before at the end of list
		this.listContainer.appendChild(this.ghostNode);
		this.isGhostInList = true;
		this._removeEmptyGhost();
	},

	_removeGhost: function() {
	// summary: remove ghostnode
		if(this.isGhostInList) {
			this.listContainer.removeChild(this.ghostNode);
			this.isGhostInList = false;
			this._insertEmptyGhost();
		}
	},

	_insertEmptyGhost: function() {
	// summary: show empty Node
		if(!this._hasElements()) {
			try {
				this.listContainer.appendChild(this.emptyNode);
			} catch(e) {
				// IE - can not add TD in an empty TR
				if (this.emptyNode.nodeName.match(/TD/i)) {
					var table = this.listContainer.parentNode.parentNode;
					table.outerHTML = table.outerHTML.replace(/<\/TR>/i, this.emptyNode.outerHTML + '$1');
				} else {
					new Error('droplist._insertEmptyGhost: could not append node, reason: ' + e.message);
				}
			}
		}
	},

	_insertNewGhost: function() {
	// summary: insert button for new items
		// show new Node
			try {
				this.listContainer.appendChild(this.newNode);
			} catch(e) {
				// IE - can not add TD in an empty TR
				if (this.emptyNode.nodeName.match(/TD/i)) {
					var table = this.listContainer.parentNode.parentNode;
					table.outerHTML = table.outerHTML.replace(/<\/TR>/i, this.emptyNode.outerHTML + '$1');
				} else {
					new Error('droplist._insertNewGhost: could not append node, reason: ' + e.message);
				}
			}
	},

	_removeEmptyGhost: function() {
	// summary: remove node with empty ghost (empty droplist hint)
		if(this._hasElements() && this.emptyNode.parentNode == this.listContainer){
			this.listContainer.removeChild(this.emptyNode);
		}
	},

	_createWrapper: function(/* domnode */ targetNode) {
	// summary: create a wrapper node
		var wrapper=document.createElement(this.options.wrapperNodeName);
		wrapper.className = "Wrapper";
		wrapper.ep_isWrapper = true;
		return wrapper;
	},

	_removeWrapper: function(/* domnode */ obj) {
	// summary: remove wrapper node
		obj.parentNode.removeChild(obj);
	},

	_markWrapper: function() {
	// summary: mark child nodes in listContainer as wrapper nodes
		var node = this.listContainer.firstChild;

		while(node != null) {
			if(node.nodeType == 1) {
				node.ep_isWrapper = true;
			}
			node=node.nextSibling;
		}
	},

	_listChange: function() {
	// summary: event function - droplist/change - update the emptyGhost
		if(!this._hasElements()) {
			this._insertEmptyGhost();
		}else {
			this._removeEmptyGhost();
		}
	},

	_cleanUpList: function() {
	// summary: remove all nodes which are no real nodes
			var node = this.listContainer.firstChild;
			while(node != null) {
				var deleteNode=undefined;
				if(node.nodeType != 1) {
					deleteNode = node;
				}
				node=node.nextSibling;
				if(deleteNode) {
					this.listContainer.removeChild(deleteNode);
				}
			}
	},

	_hasElements: function() {
	// summary: returns if childNodes exist
		return this.listContainer.hasChildNodes();
	}

};