/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.presentation.widget.Createobjectlink $Revision: 1.11.4.4 $

*/
dojo.provide('epages.cartridges.de_epages.presentation.widget.Createobjectbutton');
dojo.require("epages.widget.LocalizedWidget");
dojo.require("epages.io.json");
dojo.require("epages.cartridges.de_epages.presentation.icon");

dojo.declare(
	"epages.cartridges.de_epages.presentation.widget.Createobjectbutton",
	[epages.widget.LocalizedWidget],
	{
		templateString:     '',
		templatePath: 			dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/Createobjectbutton.html'),
		translationName: 		dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/translation'),

		className:					'',
		objectId:						'',
		description:				'',
		iconFile:						'',
		url :								'?',
		undoWidget:					'',
		languageId:					undefined,
		formNode:						null,
		dialogTitle:				'',
		focusFirstInput:		false,

		_errorInputs:				null,

		postMixInProperties: function() {
			if(!this.languageId) {
				this.languageId = epages.vars.Locale.languageId;
			}

			if(this.iconFile == ''){
				this.iconFile = epages.themeUrl('/images/ico_s_add.png');
			}

			if (this.description == "") {
				this.description = this.translate('New');
			}

			this.focusFirstInput = Boolean(this.focusFirstInput);
			this.inherited("postMixInProperties", arguments);
		},

		postCreate: function () {
			this.inherited("postCreate", arguments);
			this._errorInputs = [];
			// add events to buttons
			this.connect(this.domNode, "onclick", "createObject");
		},

		createObject: function() {
			var undoWidget = $$(this.undoWidget);
			if(undoWidget && undoWidget.existsChanges()) {
				this._confirmSaveChanges();
				return;
			}
			var _this = this;
			var doCreateObject = dojo.hitch(this, function(params, button) {
				var json = new epages.io.Json();
				if (params == null){ 
					params = {};
				}
				params = dojo.mixin({
					ChangeAction: 'NewObject',
					ObjectID: 		this.objectId,
					Class: 				this.className,
					ViewAction:   'JSONGet',
					Attributes:   'ObjectID,ClassAlias=Class.Alias,IsVisible,Position,Name=NameOrAlias', //isProduct is not set for Forum/Guestbook...
					ErrorAction:  'JSONViewResponse'
				}, params);
				if (button) {
							button._modalDialogWidget.hide();
						}
				var data = json.loadAsync(this.url, dojo.hitch(this,function(result){
					if(result.error){
						if (button) {
							button._modalDialogWidget.show();
						}
						for( var i=0,iLength=this._errorInputs.length ; i<iLength ; i++ ) {
							$E(this._errorInputs[i]).removeClassName('DialogError');
						}
						this._errorInputs = [];
						var data = result.error.data;
						for( var i=0,iLength=data.Errors.length ; i<iLength ; i++ ) {
							var error = data.Errors[i];
							// find element for error by name
							var input = parent.dojo.query('*[name=' + error.Parameter + ']', button._modalDialogWidget.domNode);
							if (input.length > 0) {
								input = input[0];
								epages.topDojo.publish("uimessage/show", ["", this.translate(error.Reason), "Bubble", {
									typeClass: "Warning",
									sizeClass: "Small",
									element: input,
									attachNode: input.parentNode,
									baseNode: button._modalDialogWidget.domNode
								}]);
								$E(input).addClassName('DialogError');
								this._errorInputs.push(input);
							}
						}
					}
					else{
						if (button) {
							button._modalDialogWidget.destroyRecursive();
						}
						window.setTimeout(function() {
							parent.dojo.publish('Createobjectbutton/objectCreated', [{ result: result }]);
						}, 100);
					}
				}), params);
			});
			var json = new epages.io.Json();
			json.loadAsync(this.url, function(result) {
				if (result.data.snippet !== null) {
					// display create object form
					epages.topDojo.require('epages.widget.Modaldialog');
					var newDiv = epages.topWindow.document.createElement('div');
					epages.topWindow.document.getElementsByTagName('body')[0].appendChild(newDiv);
					this._createObjectWidget = new epages.topWindow.epages.widget.Modaldialog({
						// we have to focus the input ourselves, so disable default autofocus
						autofocus : !_this.focusFirstInput,
						options:{
							title : _this.dialogTitle,
							content: result.data.snippet
						},
						buttons: [{
							label: _this.translate('Insert'),
							cssClass: "Active",
							onclick: function() {
								// get values from form
								var formNode = this._modalDialogWidget.containerNode.getElementsByTagName('form');
								if(formNode){
									var params = {};
									if (formNode.length > 0) {
										formNode = formNode[0];
										for( var i=0,iLength=formNode.elements.length ; i<iLength ; i++ ) {
											var element = formNode.elements[i];
											if ((element.nodeName.match(/INPUT/i) && element.type.match(/RADIO|TEXT|CHECKBOX/i)) ||
												element.nodeName.match(/TEXTAREA|SELECT/i)) {
												params[element.name] = element.value;
											}
										}
									}
									doCreateObject(params, this);
								}
							}
						}, {
							label: _this.translate('Cancel'),
							onclick: function() {
								this._modalDialogWidget.destroyRecursive();
								parent.dojo.publish('Createobjectbutton/closed');
							}
						}]
					}, newDiv);
					this._createObjectWidget.show();
				} else {
					// empty snippet - no dialog
					// this happens when no template is defined for this class
					doCreateObject();
				}
			},
			{
				'ViewAction': 'JSONSnippet',
				'TemplateAction': 'CreateObjectSnippet',
				'TemplateLanguageID': this.languageId,
				'ObjectID': this.objectId,
				'Class': this.className,
				'DontFail': '1'
			});
		},

		_confirmSaveChanges: function(){
			var me = this;
			parent.dojo.publish("uimessage/show", [
				this.translation.get("SaveQuestion"),
				this.translation.get("CreateObjectDescription"),
				'Dialog',
				{
					titleBar: this.translation.get("ChangeOnContent"),
					sizeClass:"Large",
					typeClass:"Warning",
					buttons: [{
						label: this.translation.get("Save"),
						cssClass: 'Active',
						onclick: function() {
							this._modalDialogWidget.destroyRecursive();
							//if Save was clicked, undowidget saves changes
							$$(me.undoWidget).sendChanges();
							$$(me.undoWidget).undoObject.clear();
							//perform action
							me.createObject();
						}
					},
					{
						label: this.translation.get("Cancel"),
						onclick: function() {
							this._modalDialogWidget.destroyRecursive();
						}
					}]
				}
			]);
		}

	}
);
