/*
	Copyright (c) 2006-2008, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.presentation.widget.History $Revision: 1.4.8.6 $

*/
dojo.provide("epages.cartridges.de_epages.presentation.widget.History");
dojo.require("epages.widget");
dojo.require("epages.io.json");
dojo.require('epages.html');
dojo.require('epages.format');
dojo.require('epages.widget.LocalizedWidget');

dojo.declare(
	"epages.cartridges.de_epages.presentation.widget.History",
	[epages.widget.LocalizedWidget],
	{
		objectId           : '',
		url                : '?',
		sidebarTabsId      : 'RightSidebarTabs',
		sidebarId          : 'RightSidebar',
		showHistory        : false,
		favoritesWidgetId  : 'favoritesWidget',

		_history           : undefined,
		_sidebarTabs       : undefined,
		_favoritesWidget   : undefined,
		_sidebar           : undefined,

		templatePath       : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/History.html'),
		imagePath          : epages.themeUrl('images'),
		translationName    : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/translation'),

		postCreate  : function() {
			this.inherited("postCreate", arguments);
			dojo.subscribe('Sidebar/selectTab', this, '_onTabSelect');
			if($$(this.sidebarId)){
				this._sidebar = $$(this.sidebarId);
			}
			if($$(this.sidebarTabsId)){
				this._sidebarTabs = $$(this.sidebarTabsId);
				var tab = this._sidebarTabs.currentTab();
				//check if tab is selected and sidebar is visible before loading
				if(tab && this._sidebar.isVisible){
					this._onTabSelect(tab);
				}
			}else{
				this._loadHistory();
				this._updateLayout();
			}
			if($$(this.favoritesWidgetId)){
				this._favoritesWidget = $$(this.favoritesWidgetId);
			}
		},

		_onTabSelect:function(tab){
			if(tab && tab.menu == 'history' && this._history == undefined){
				this._loadHistory();
				this._updateLayout();
			}
		},

		_loadHistory: function(){
			var currentHistory = this._sendRequest('JSONGetHistory');
			if(currentHistory){
				this._history = currentHistory.data;
				this.containerNode.innerHTML = '';
				for( var i=0,iLength=this._history.length ; i<iLength ; i++ ){
					var item = this.itemNode.cloneNode(true);
					item.style.display = '';
					var linkNode = epages.html.getElementsByClassName('ItemLink',item)[0];
					linkNode.innerHTML = epages.format.scrunch(this._history[i].Name,30, 0.4);
					linkNode.title = this._history[i].Name;
					linkNode.href = this._history[i].URI;
					if(i == 0){
						dojo.addClass(linkNode,'Bold');
					}
					var addNode = epages.html.getElementsByClassName('ItemAddNode',item)[0];
					this.connect(addNode, 'onclick', 'addHistoryToFavorites');
					this._history[i].itemNode = item;
					this._history[i].addNode = addNode;
					this.containerNode.appendChild(item);
				}
			}
		},

		_updateLayout: function(){
			if(this._history && this._history.length == 0){
				this.emptyNode.style.display = '';
				this.deleteNode.style.display = 'none';
				this.containerNode.style.display = 'none';
			}
			else{
				this.emptyNode.style.display = 'none';
				this.deleteNode.style.display = '';
				this.containerNode.style.display = '';
			}
			if(this.showHistory){
				this.toggleNodeTitle.innerHTML = this.translate('HideHistory');
				this.toggleNodeHr.style.display = '';
			}
			else{
				this.toggleNodeTitle.innerHTML = this.translate('ShowHistory');
				this.deleteNode.style.display = 'none';
				this.containerNode.style.display = 'none';
				this.emptyNode.style.display = 'none';
				this.toggleNodeHr.style.display = 'none';
			}
		},


		_deleteHistory: function(){
			if(this._sendRequest('DeleteHistory')){
				this._history = [];
				this.containerNode.innerHTML = "";
				this._showHistoryEmpty();
				this._updateLayout();
			}
		},

		_showHistoryEmpty: function(){
			dojo.removeClass(this.emptyNodeHistoryEmpty, "HideElement");
			dojo.addClass(this.emptyNodeInfoHistory, "HideElement");
		},

		_showInfoHistory: function(){
			dojo.removeClass(this.emptyNodeInfoHistory, "HideElement");
			dojo.addClass(this.emptyNodeHistoryEmpty, "HideElement");
		},

		_sendRequest: function(changeAction,opt){
			var json = new epages.io.Json;
				var options = {
					'ViewAction':     'JSONViewResponse',
					'ChangeAction':   changeAction,
					'ObjectID':       this.objectId
				};
				if(opt){
					dojo.mixin(options,opt);
				}
				var result = json.loadSync(this.url,options);
				if(result.error){
					return false;
				}
				return result;
		},

		_toggleHistory: function(){
			var toggle = false;
			if(this.showHistory == true){
				if(!this._sendRequest('HideHistory')){
					return;
				}
				this._showHistoryEmpty();
			}
			else {
				if(!this._sendRequest('ShowHistory')){
					return;
				}
				this._showInfoHistory();
			}
			this.showHistory = !this.showHistory;
			this._updateLayout();
		},

		addHistoryToFavorites:function(evt){
			originalTarget = evt.originalTarget || evt.srcElement;
			var history = undefined;
			var index = undefined;
			for( var i=0,iLength=this._history.length ; i<iLength ; i++ ){
				if(this._history[i].addNode == originalTarget){
					history = this._history[i];
					index = i;
				}
			}
			if(history){
				if(this._favoritesWidget){
					this._favoritesWidget.addHistoryToFavorites(history);
				}
				else{
					this._sendRequest('JSONCopyHistoryToFavorite',{EventHistoryID: history.EventHistoryID});
				}
			}
		}
});

