/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.presentation.widget.Menucontroller $Revision: 1.30.2.5 $

*/
dojo.provide("epages.cartridges.de_epages.presentation.widget.Menucontroller");
dojo.require("epages.lang.hitch");
dojo.require('epages.template');
dojo.require('epages.widget');
dojo.require('epages.lang.hash');

dojo.declare(
	"epages.cartridges.de_epages.presentation.widget.Menucontroller",
	[dijit._Widget], {
		isMenuBar          : true,
		absoluteBase       : '',

		/**
		 * menu hierachy props
		 */
		_children          : {},
		_menuAliasMap      : {},
		_childrenTree      : null,
		_temporaryContainer: null,			// prepare initial nodes outside dom

		/**
		 * menu behavior props
		 */
		_openMenuLayers    : $H(),			// key: menu depth , value: array with content open dom nodes
		_currentMenuDepth  : -1,				// current depth of open layers
		_openLayerTimeout  : undefined,	// timeout-id for the next sub menu to open
		_closeLayerTimeout : undefined,	// timeout-id for the next sub menu to close
		_TIMEOUT_DUARATION : 500,
		/**
		 * templates
		 */
		menuTemplateString :
				'<div class="Layer"><div class="PropertyContainer ItemContainer"></div></div>' +
				'<div class="LayerFooter"><div class="PropertyContainer"></div></div><div class="BackgroundIframe"></div>',

		menuItemString: '<(%tag%) href="(%url%)" (%onclick%) (%onmouseover%) (%onmouseout%) class="ClickNode"><span class="Container"><span class="Icon" (%icon%)>(%label%)<span class="OpenSubMenu"></span></span></span></(%tag%)><(%tag%) hidefocus="true" class="ClickNode LinkLayer" href="(%url%)" (%onclick%) (%onmouseover%) (%onmouseout%) (%id%)></(%tag%)>',

		menuIconString: 'style="background-image: url((%iconSrc%)); background-repeat: no-repeat"',

		separatorString: '<div></div>',

		postCreate  : function() {
			this.inherited("postCreate", arguments);
			document.getElementsByTagName("body")[0].appendChild(this.domNode);
			this._temporaryContainer = document.createElement("div");
			dojo.publish('menucontroller/postCreate');
			this.connect(this.domNode, "onmouseout", "onMouseOut");
			this.connect(this.domNode, "onmouseup", "onMouseUp");
			this.connect(document.body, "onmouseup", "onMouseUpBody");
			this.connect(window, "onkeypress", "onKeypress");
			this.connect(document.body, "onkeypress", "onKeypress");
			dojo.subscribe(this.id+"/enableMenuitem", this, "onEnableMenuItem");
			dojo.subscribe(this.id+"/disableMenuitem", this, "onDisableMenuItem");
			dojo.subscribe(this.id+"/setUri", this, "onSetUri");
		},

		registerChildren: function( childrenArray, depth) {
			if(depth === undefined){
				depth = 0;
			}
			for( var i=0,l=childrenArray.length ; i<l ; i++ ) {
				var child = childrenArray[i];
				if(child !== undefined && child != null && child.objectId !== undefined) {
					this._children[child.objectId] = child;
					if(child.menualias) {
						this._menuAliasMap[child.menualias] = child.objectId;
					} else {
						console.warn("child.menualias not defined for menuitem '"+child.title+"' (objectId="+child.objectId+") in "+this.declaredClass);
					}
					child.menu_depth = depth;
					if (child.children !== undefined) {
						this.registerChildren(child.children, depth+1);
					}
				}
			}
			this._childrenTree = childrenArray; // register outer array
		},

		updateChildren: function(childrenArray) {
			for( var i=0,l=childrenArray.length ; i<l ; i++ ) {
				var child = childrenArray[i];
				if (child !== undefined  && child != null && child.objectId !== undefined && this._children[child.objectId]) {
					dojo.mixin(this._children[child.objectId], child);
				}
			}
		},

		childrenTree: function () {
			return this._childrenTree;
		},

		setChildren: function(childrenArray, parentNode) {
			var tempDiv = document.createElement("div");
			for( var i=0,l=childrenArray.length ; i<l ; i++ ) {
				var child_org = childrenArray[i];
				var child = (child_org) ? this._children[child_org.objectId] : null;			// use copy (for easy data acces)
				if(child!=null) { // check whether current element exsits or not
					var menuNode = undefined;
					if (child.children!= null) {
						child.children = dojo.filter(child.children, function(el) { return el !== undefined && el != null; });
					}
					var hasChildren = (child.children!= null && child.children.length);

					menuItem = tempDiv.cloneNode(true);

					if(parentNode !== undefined) {
						menuItem.className="MenuItem EnableHover";
					} else {
						menuItem.className="MenuItem EnableHover Main";
					}

					menuItem.id = (child.objectId === undefined) ? undefined : 'MenuItem_'+child.objectId;


					if(child.objectId === undefined) {
						console.warn("MenuItem ObjectID undefined in "+this.declaredClass);
					}

					if(child.type=="Separator") {
						menuItemString = this.separatorString;
						menuItem.className+=" Seperator";
					} else {
						// create MenuItem
						if (child.uri !== undefined && this.absoluteBase != '' && child.uri.match(/^(?![a-zA-Z]+\:\/\/|\/)/)) {
							// transform relative urls to absolute if absoluteBase is set
							child.uri = this.absoluteBase + child.uri;
						}
						var uri = (child.uri !== undefined) ? child.uri : "";

						if(child.disabled) {
							uri = "javascript: void(0);";
						}

						var menuItemOptions = {
							tag : (uri != "" ) ? 'a' : 'span',
							label : child.title,
							disabled : child.disabled,
							url : uri,
							icon : '',
							onclick: '',
							id : (child.menualias) ? 'id='+child.menualias : ''
						};

						if(!child.menualias) {
							console.warn('menu alias missing for "'+child.title+'" in '+this.declaredClass);
						}

						menuItemOptions['onmouseover'] = 'onmouseover="$$(\''+this.id+'\').menuOnMouseOver(\''+child.objectId+'\',this);"';
						menuItemOptions['onmouseout']  = 'onmouseout="$$(\''+this.id+'\').menuOnMouseOut(\''+child.objectId+'\', this);"';

						if(child.iconSrc) {
							iconTemplate = $TMPL(this.menuIconString, {iconSrc : child.iconSrc});
							menuItemOptions['icon'] = iconTemplate.generateOutput();
						}

						if (hasChildren) {
							menuItem.className+=" HasChildren";
							menuItemOptions['onclick'] = 'onclick="$$(\''+this.id+'\').menuOnClick(\''+child.objectId+'\', this); return false"';
						}

						menuItemTmpl = $TMPL(this.menuItemString, menuItemOptions, false, false);

						var menuItemString = menuItemTmpl.generateOutput();
					}

					menuItem.innerHTML = menuItemString;

					var onClickProperty = undefined;
					if(child.onClick!=null) {
						onClickProperty = child.onClick;
					}
					if(typeof onClickProperty == 'function') {
						var llNode = dojo.query(".LinkLayer", menuItem)[0];
						if(typeof llNode != 'undefined') {
								llNode.onClickProperty = onClickProperty;
								llNode.onclick = dojo.hitch(llNode, function() {
									if(!this.parentNode.className.match("Disabled")) {
										this.onClickProperty();
									}
								});
						}
					}

					var containerNode = undefined;
					if(parentNode !== undefined) {
						containerNode = epages.html.getElementsByClassName("ItemContainer", parentNode)[0];
					} else {
						containerNode = this._temporaryContainer;
					}

					if(child.disabled) {
						dojo.addClass(menuItem,"Disabled");
					}

					if(child.isSelected) {
						dojo.addClass(menuItem,"Selected");
					}

					if(child.featureDisabled) {
						dojo.addClass(menuItem,"FeatureDisabled");
					}

					containerNode.appendChild(menuItem);

					if(hasChildren) {
						// create menu container
						menuNode = tempDiv.cloneNode(true);
						menuNode.className="Menu MenuContainer";
						if(child.menualias) {
							menuNode.className+=" "+child.menualias;
						}
						menuNode.innerHTML = this.menuTemplateString;
						menuNode.id = (child.objectId === undefined ? undefined : 'Menu_'+child.objectId);
						if(child.objectId === undefined) {
							console.warn("Menu Child ObjectID undefined in "+this.declaredClass);
						}
						menuItem.insertBefore(menuNode, menuItem.firstChild);
					}
				}
			}
			if(parentNode == undefined) {
				this.domNode.appendChild(this._temporaryContainer);
			} else {
				dojo.publish(this.id+"/"+parentNode.id+"_ready", []);
			}
		},

		_getElementFromTree:function(objectId,tree){
			var elementsWithChildren = [];
			for( var i=0,iLength=tree.length ; i<iLength ; i++ ){
				if(tree[i] && tree[i].objectId == objectId){
					return tree[i];
				}
				if(tree[i] && tree[i].children && tree[i].children.length != 0){
					elementsWithChildren.push(tree[i]);
				}
			}
			for( var j=0,jLength=elementsWithChildren.length ; j<jLength ; j++ ){
				var elm = this._getElementFromTree(objectId,elementsWithChildren[j].children);
				if(elm != null) {
					return elm;
				}
			}
			return null;
		},

		onMouseOut: function(evt) {
		// summary: menucontroller mouse out
			if(this._closeLayerTimeout) {
				window.clearTimeout(this._closeLayerTimeout);
			}
			var _this = this;
			this._closeLayerTimeout = window.setTimeout( function() {
				_this.closeOpenMenus(-1);
				_this._currentMenuDepth = -1;
			}, this._TIMEOUT_DUARATION);
		},

		onKeypress: function(evt) {
			if(evt.keyCode == dojo.keys.ESCAPE) {
				this.onMouseUpBody();
			}
		},

		onMouseUpBody: function() {
			this.closeOpenMenus(-1);
			this._currentMenuDepth = -1;
		},

		onMouseUp: function(evt) {
			dojo.stopEvent(evt);
		},

		menuOnClick: function(objectId,node){
			this.onOpenMenu(objectId,node, "click");
		},

		menuOnMouseOver: function(objectId,node){
			this.onOpenMenu(objectId,node,"mouseover");
		},

		menuOnMouseOut: function(objectId,node) {
			// hide submenu after leaving parent menuitem
			var menuItem = this._getElementFromTree(objectId,this._childrenTree);
			if(menuItem){
				var menuItemNode = $('MenuItem_'+objectId);
				var depth = menuItem.menu_depth;
				if(this._openMenuLayers.get(depth) == menuItemNode) {
					var _this = this;
					this._closeLayerTimeout = window.setTimeout( function() {
						_this.closeOpenMenus(depth-1);
					}, this._TIMEOUT_DUARATION);
				}
			}

			// clear open layer timeout
			if(this._openLayerTimeout) {
				window.clearTimeout(this._openLayerTimeout);
			}
		},

		onEnableMenuItem : function(/* string */ menualias) {
			var objectId = this._menuAliasMap[menualias];
			if(objectId) {
				delete this._children[objectId].disabled;
				var menuItem = $('MenuItem_'+objectId);
				if(menuItem) {
					dojo.removeClass(menuItem, 'Disabled');
				}
			}
		},

		onDisableMenuItem : function(/* string */ menualias) {
			var objectId = this._menuAliasMap[menualias];
			if(objectId) {
				this._children[objectId].disabled = true;
				var menuItem = $('MenuItem_'+objectId);
				if(menuItem) {
					dojo.addClass(menuItem, 'Disabled');
				}
			}
		},

		onSetUri : function(/* string */ menualias, /* string */ uri) {
			var objectId = this._menuAliasMap[menualias];
			if(objectId) {
				this._children[objectId].uri = uri;
				var menuItem = $('MenuItem_'+objectId);
				var aTags = dojo.query("a", menuItem);
				dojo.forEach(aTags, function(el) {
					el.href=uri;
				});
			}
		},

		onOpenMenu: function(objectId,node, eventType) {
			var menuItem = this._getElementFromTree(objectId,this._childrenTree);
			if(menuItem){
				var menuNode = $('Menu_'+objectId);
				var menuItemNode = $('MenuItem_'+objectId);
				var depth = menuItem.menu_depth;
				this.clearTimeouts();
				if(menuNode){
					var containerNode = epages.html.getElementsByClassName("ItemContainer", menuNode)[0];
					// create
					if(containerNode.childNodes.length == 0){
						this.setChildren(menuItem.children,menuNode);
					}
					// show
					if(this._currentMenuDepth >= depth || eventType == "click") {					// switch to menu items of same level asap
						this.showSubMenu(menuItemNode,depth);
					} else {																			// delay switch to deeper menu items
						var _this = this;
						this._openLayerTimeout = window.setTimeout( function() {
							_this.showSubMenu(menuItemNode, menuItem.menu_depth);
						}, this._TIMEOUT_DUARATION);
					}
				} else {
						var _this = this;
						this._openLayerTimeout = window.setTimeout( function() {
							_this.closeOpenMenus(depth);
						}, this._TIMEOUT_DUARATION);
				}
			} else{
				console.warn("MenuItem undefined in "+this.declaredClass);
			}
		},

		clearTimeouts: function() {
			if(this._openLayerTimeout) {
				window.clearTimeout(this._openLayerTimeout);
			}
			if(this._closeLayerTimeout) {
				window.clearTimeout(this._closeLayerTimeout);
			}
		},

		showSubMenu: function(node, depth) {
			this.closeOpenMenus(depth);												// close all menus of same or deeper level
			this.showMenuNode(node);													// show the sub menu
			this._openMenuLayers.set(depth.toString(), node); // register menu node in the open menu hash
			this._currentMenuDepth = depth;										// change current depth
		},

		closeOpenMenus: function(depth) {
		// summary: closes all open menus of the specified and deeper levels
			var _this = this;
			this._openMenuLayers.each( function(pair) {
				var pk = parseInt(pair.key);
				if(pk >= depth) {
					var lastOpenMenuNode = _this._openMenuLayers.get(pair.key);
					if(lastOpenMenuNode !== undefined) {	// show last open menu node
						_this.hideMenuNode(lastOpenMenuNode);
						_this._openMenuLayers.set(pair.key, undefined);
					}
				}
			});
		},

		disableMenuItem : function(/* string */id, /* boolean? */ exceededFeatureLimet) {
			console.warn("deprecated function disableMenuItem() used  in "+this.declaredClass, "Use dojo.publish('<menucontrollerid>/disableMenuitem', []); instead");
			var miNode=$(id);
			if(miNode) {
				dojo.addClass(miNode, "Disabled");
				if(exceededFeatureLimet) {
					dojo.addClass(miNode, "FeatureDisabled");
				}
			}
		},

		enableMenuItem: function(/* string */id) {
			console.warn("deprecated function enableMenuItem() used in "+this.declaredClass, "Use dojo.publish('<menucontrollerid>/enableMenuitem', []); instead");
			var miNode=$(id);
			if(miNode) {
				dojo.removeClass(miNode, "Disabled");
			}
		},

		showMenuNode: function(node) {
			dojo.addClass(node, "IsOpen");
		},

		hideMenuNode: function(node) {
			dojo.removeClass(node, "IsOpen");
		}
	}
);
