/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

  epages.cartridges.de_epages.presentation.widget.searchform $Revision: 1.2.8.6 $

*/
dojo.provide("epages.cartridges.de_epages.presentation.widget.SearchForm");
dojo.require("epages.browser");
dojo.require("epages.cssfix");
dojo.require("dojo.fx");
dojo.require("dojox.fx");
dojo.require("dojo.fx.easing");
dojo.require("epages.io.translation");

dojo.declare("epages.cartridges.de_epages.presentation.widget.SearchForm", [dijit._Widget], {

	closeIcon:                   (epages.themeUrl("images") + "/layer_img_closebutton.png"),

	_showSearchFormTranslation:  null,
	_hideSearchFormTranslation:  null,

	spacerNode: null,
	buttonNode: null,

	showAtStartup: false,
	_toggleState: null,
	animationActive: false,

	_recalcFormWidth: function() {
		this.domNode.style.width = this.spacerNode.offsetWidth + "px";
	},

	_resetHiddenPosition: function() {
		if(this._toggleState && !this.animationActive) {
			this.domNode.style.left = dojo.body().offsetWidth + "px";
		}
	},

	_createSearchFormExtension: function(showAtStartup) {

		var presentationTranslation = new epages.io.Translation(dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget','templates/translation'), "auto");
		this._showSearchFormTranslation = presentationTranslation.get("ShowSearch");
		this._hideSearchFormTranslation = presentationTranslation.get("HideSearch");

		dojo.style(this.domNode, {
			position 	: "absolute",
			top 			: "10px",
			left 			: showAtStartup ? "10px" : (dojo.body().offsetWidth + 10) + "px"
		});

		var closeNode = document.createElement("div");
		closeNode.className = "SearchFormCloseIcon EnableHover";
		closeNode.title = this._hideSearchFormTranslation;
		this.domNode.appendChild(closeNode);

		this.connect(closeNode, "click", "_toggleSearchForm");

		var buttonNode = document.createElement("div");
		buttonNode.className = "SearchFormButton EnableHover" + (showAtStartup ? " HideElementSoft": "");

		buttonNode.innerHTML = '<i class="Sprite ico_s_search"></i><a href="javascript:void(0)">' + this._showSearchFormTranslation + '</a>';

		this.domNode.parentNode.insertBefore(buttonNode, this.domNode);

		var spacerNode = document.createElement("div");
		spacerNode.className = "SearchFormSpacer";
		spacerNode.innerHTML = " ";

		if(showAtStartup) {
			spacerNode.style.height = (this.domNode.offsetHeight + (epages.Browser.engine == "MSIE" ? 0 : 6)) + "px"
		}

		this.domNode.parentNode.insertBefore(spacerNode, buttonNode);

		this.spacerNode = spacerNode;
		this.buttonNode = buttonNode;

		this._recalcFormWidth();
	},

	_toggleSearchForm: function() {
		if(this.animationActive) {return}
		this.animationActive = true;
		if(this._toggleState) {
			this.domNode.style.visibility = "visible";
			dojox.fx.wipeTo({node: this.spacerNode, height: (this.domNode.offsetHeight + (epages.Browser.engine == "MSIE" ? 0 : 6)), duration: 350, easing: dojo.fx.easing.easeOut}).play();
			dojo.fx.slideTo({node: this.domNode, top: 10, left:10, duration: 350, easing: dojo.fx.easing.easeOut}).play();
			dojo.fadeOut({node: this.buttonNode, duration:350, easing: dojo.fx.easing.easeOut}).play();
			setTimeout(dojo.hitch(this, function() {
				this.animationActive = false;
				this._toggleState = !this._toggleState;
				this.buttonNode.className += " HideElementSoft";
				try{dojo.query("input[type=text].SearchString", this.domNode)[0].focus()}catch(e){};
				this._recalcFormWidth();
			}), 350);
		}else {
			dojox.fx.wipeTo({node: this.spacerNode, height: (epages.Browser.engine == "MSIE" ? 0 : 6), duration: 350, easing: dojo.fx.easing.easeOut}).play();
			dojo.fx.slideTo({node: this.domNode, top: 10, left:(dojo.body().offsetWidth + 10), duration: 350, easing: dojo.fx.easing.easeOut}).play();
			dojo.style(this.buttonNode, "opacity", 0);
			dojo.removeClass(this.buttonNode, "HideElementSoft");
			setTimeout(dojo.hitch(this, function() {
				dojo.fadeIn({node: this.buttonNode, duration:350, easing: dojo.fx.easing.easeOut}).play();
			}), 50);
			setTimeout(dojo.hitch(this, function() {
				this.domNode.style.visibility = "hidden";
				this._toggleState = !this._toggleState;
				this.animationActive = false;
			}), 350);
		}
	},

	_checkDialogError: function() {
		if(this.domNode.innerHTML.match("DialogError")) {
			$$(dojo.query(".ToolbarButtons input.ButtonContainer", this.domNode)[0].id).disable();
		}else {
			$$(dojo.query(".ToolbarButtons input.ButtonContainer", this.domNode)[0].id).enable();
		}
	},

	postCreate: function() {
		this.inherited("postCreate", arguments);
		this._createSearchFormExtension(this.showAtStartup);

		this._toggleState = !this.showAtStartup;

		this.connect(window, "resize", "_resetHiddenPosition");
		this.connect(window, "resize", "_recalcFormWidth");
		this.connect(dojo.body(), "click", "_recalcFormWidth");

		this.connect(this.domNode, "keydown", "_checkDialogError");
		this.connect(this.domNode, "keyup", "_checkDialogError");
		this.connect(dojo.body(), "click", "_checkDialogError");
		dojo.addOnLoad(dojo.hitch(this, function() {
			var inputElements = dojo.query("input[type=text]", this.domNode);
			for( var i=0,iLength=inputElements.length ; i<iLength ; i++ ) {
				this.connect(inputElements[i], "change", "_checkDialogError");
			}
		}));

		this.connect(this.buttonNode, "click", "_toggleSearchForm");

		dojo.addOnLoad(dojo.hitch(this, function(){setTimeout(dojo.hitch(this, function() {
			this._recalcFormWidth();
		}), 1)}));
		dojo.addOnLoad(dojo.hitch(this, function(){setTimeout(dojo.hitch(this, function() {
			this._recalcFormWidth();
		}), 100)}));
	}

});