/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.presentation.widget.Sidebar $Revision: 1.19.62.3 $

*/

dojo.provide("epages.cartridges.de_epages.presentation.widget.Sidebar");
dojo.require("epages.widget.LocalizedWidget");
dojo.require("epages.event.adjustonresize");
dojo.require('dijit._Container');

dojo.declare(
	"epages.cartridges.de_epages.presentation.widget.Sidebar",
	[epages.widget.LocalizedWidget, dijit._Contained], {
		side              : "",
		width             : "",
		isVisible         : '',
		isOverlay         : false,
		title             : "",
		displayTitlebar   : true,
		minimizeOnHide    : false,
		url               : '?',
		userId            : '',
		pageTypeId        : '',

		WIDTH_OFFSET      : 1, // Integer; width offset (borders, margins, paddings)

		templatePath    : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', "templates/Sidebar.html"),
		translationName : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', "templates/translation"),
		imagePath       :epages.themeUrl('images'),

		postCreate: function() {
			this.inherited("postCreate", arguments);

			this.isVisible=epages.string.toBoolean(this.isVisible);
			this.minimizeOnHide=epages.string.toBoolean(this.minimizeOnHide);
			this.displayTitlebar=epages.string.toBoolean(this.displayTitlebar);
			this.setTitlebar(this.displayTitlebar);

			// stretch to page length
			this.domNode.style.height="100%";
			this.setWidth(parseInt(this.width));
			this._appendEvents();
			if(this.minimizeOnHide) {
				dojo.addClass(this.domNode, "Minimize");
			} else {
				this.domNode.style.display="none";
			}
		},

		setTitlebar: function(displayTitlebar) {
			if(displayTitlebar) {
				dojo.addClass(this.domNode, "WithTitlebar");
				this.displayTitlebar=true;
			} else {
				dojo.removeClass(this.domNode, "WithTitlebar");
				this.displayTitlebar=false;
			}
		},

		_appendEvents: function() {
			this.connect(this.closeButtonNode, 'onclick', 'hide');
			this.connect(this.minimizeNode,    'onclick', 'showLastOverlayState');
		},

		setWidth: function(width) {
			if(isNaN(width)){
				console.warn('Attribute "width" is not a number in '+this.declaredClass+'. Couldn\'t calculate new width');
			}else {
				this.width=width;
				this.domNode.style.width=this.width+"px";

				if(!this.isOverlay && this.isVisible){
					dojo.publish(this.id+"/afterSetWidth", [{
						width    : this.width+this.WIDTH_OFFSET,
						side     : this.side
					}]);
				}
			}
		},

		setTitle: function(title) {
				this.title=title;
				this.titleNode.innerHTML=title;
		},

		sendShow: function () {
			if(this.userId != '') {
				var json = new epages.io.Json;
				var options = {
					'ViewAction':     'JSONViewResponse',
					'ChangeAction':   'Save',
					'ObjectID':       this.userId,
					'ShowSidebar':    this.isVisible ? 1 : 0
				};
				if (!this.isVisible && this.pageTypeId != '') {
					options.ChangeAction = 'SaveSidebar';
					options.PageTypeID = this.pageTypeId;
				}
				json.loadAsync(this.url, function (result) {},options);
			}
		},

		show: function(/*Boolean?*/ overlay, startUp) {
			this.isOverlay = (overlay==true) ? true : false;
			this.isVisible = true;
			// display changes
			this.domNode.style.display="";
			dojo.removeClass(this.domNode, "Minimize");
			if(!startUp){ 
				this.sendShow();
			}
			this._toggleOverlayDisplay();
			dojo.publish(this.id+"/afterShowSidebar", [{
				width    : (this.isOverlay) ? 0 : this.width+this.WIDTH_OFFSET,
				side     : this.side
			}]);
		},

		hide: function() {
			this.isVisible=false;
			if(this.minimizeOnHide) {
				dojo.addClass(this.domNode, "Minimize");
			} else {
				this.domNode.style.display="none";
			}
			this.sendShow();
			dojo.publish(this.id+"/afterShowSidebar", [{
				width    : 0,
				side     : this.side
			}]);
		},

		toggleView: function() {
			if(this.isVisible){
				this.hide();
			}else{
				this.show(this.isOverlay);
			}
		},

		_toggleOverlayDisplay: function() {
			this.domNode.style.height=(this.isOverlay) ? "auto" : "100%";
			if(this.isOverlay) {
				dojo.addClass(this.domNode,"Unpinned");
				dojo.removeClass(this.domNode,"Pinned");
			} else {
				dojo.addClass(this.domNode,"Pinned");
				dojo.removeClass(this.domNode,"Unpinned");
			}
		},

		showLastOverlayState: function() {
				this.show(this.isOverlay);
		}
	});
