/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.presentation.widget.SidebarTabs $Revision: 1.13.62.6 $

*/
dojo.provide("epages.cartridges.de_epages.presentation.widget.SidebarTabs");
dojo.require("epages.widget");
dojo.require("epages.string");
dojo.require("epages.io.json");

dojo.declare(
	"epages.cartridges.de_epages.presentation.widget.SidebarTabs",
	[epages.widget.LocalizedWidget],
	{
		menu: '',
		forceHelp: '',
		url: '?',
		userId: '',
		pagetypeId: '',
		sideBarMenu: '',
		sideBarId: '',

		_sidebar: undefined,
		_tabs: undefined,
		_lastTab: undefined,
		_tabSelectedByClick: false,

		templatePath:  dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', "templates/SidebarTabs.html"),
		translationName : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', "templates/translation"),

		postMixInProperties: function() {
			this.inherited("postMixInProperties", arguments);
			this._tabs = [];
			this.forceHelp = epages.string.toBoolean(this.forceHelp);
			this._sidebar = $$(this.sideBarId);
		},

		postCreate  : function() {
			this.inherited("postCreate", arguments);
			// make a copy of childNodes
			var children = [];
			dojo.forEach(this.srcNodeRef.childNodes, function (el) { children.push(el); });

			var tab = true, cTab = -1, enableFirst = true;

			for( var i=0,l=children.length ; i<l ; i++ ) {
				var node = children[i];
				if (node.tagName == 'DIV') {
					if (tab) {
						cTab++;
						this.tabNode.appendChild(node);
						this._tabs[cTab] = { tab : node, menu : node.getAttribute('ep_menu')};
						this.connect(node,'click', '_clickTab');
						if ($$(this.sideBarMenu) && node.getAttribute('ep_menulabel')) {
							var caller = function (me, node) {
								return function () {
									me._clickTab({currentTarget:node});
									if(me._sidebar && !me._sidebar.isVisible){
										me._sidebar.show();
									}
								};
							};
							var menuItem  = new dijit.MenuItem({
								label: node.getAttribute('ep_menulabel'),
								onClick: caller(this, node)
							});
							$$(this.sideBarMenu).addChild(menuItem);
							if (node.getAttribute('ep_menuicon')){
								menuItem.iconNode.style.background = 'url('+node.getAttribute('ep_menuicon')+') no-repeat top left';
							}
						}
					} else {
						this._tabs[cTab].block = node;
						if(this.forceHelp && this._tabs[cTab].menu == 'help' ||
							 !this.forceHelp  && this.menu == this._tabs[cTab].menu
						) {
							this._selectTab(this._tabs[cTab].tab, node);
							enableFirst = false;
						} else {
							node.style.display = 'none';
						}
						this.blockNode.appendChild(node);
					}
					tab = !tab;
				}
			}
			if (enableFirst){
				this._selectTab(this._tabs[0].tab, this._tabs[0].block);
			}

			this.connect(window, "onbeforeunload", "_saveTab");
			if(this._sidebar !== undefined) {
				this.closeSidebarNode.style.display="";
				this.connect(this.closeSidebarNode, "onclick", "_closeSidebar");
			}
		},

		_closeSidebar: function() {
			if(this._sidebar){
				this._sidebar.hide();
			}
		},

		_clickTab: function (evt) {
			this._tabSelectedByClick=true;
			var tab = evt.currentTarget, block = undefined;
			for( var i=0,l=this._tabs.length ; i<l ; i++ )
				if (this._tabs[i].tab == tab){
					block = this._tabs[i].block;
				}
			this._selectTab(tab, block);
		},

		_selectTab: function(/* domnode */ tab, /* domnode */ block) {
			if(this._lastTab !== undefined && tab !== this._lastTab.tab) {
				// hide last selected
				dojo.removeClass(this._lastTab.tab, "Selected");
				this._lastTab.block.style.display = 'none';
			}
			// show new selected
			dojo.addClass(tab, "Selected");
			block.style.display = '';
			this._lastTab= {
				tab: tab,
				block: block,
				menu : tab.getAttribute('ep_menu')
			};

			var _this=this;
			window.setTimeout(function() {
				dojo.publish('Sidebar/changeHeight', [$$(_this.sideBarId)]);	// sidebarcontroller subscribes
			}, 10); // need a short time to display inital tab
			dojo.publish('Sidebar/selectTab',[this._lastTab]);
		},

		_saveTab: function() {
			if(this.userId != '' && this._lastTab !== undefined && this._tabSelectedByClick) {
				var json = new epages.io.Json;
				json.loadSync(this.url, {
					'ViewAction':     'JSONViewResponse',
					'ChangeAction':   'SaveSidebar',
					'ObjectID':       this.userId,
					'PageTypeID':     this.pagetypeId,
					'SidebarMenuName': this._lastTab.menu
				});
			}
		},
		selectTab: function(/*ep_menu string*/ ep_menu){
			var tab = undefined;
			var block = undefined;
			for( var i=0,iLength=this._tabs.length ; i<iLength ; i++ ){
				if(this._tabs[i].menu == ep_menu){
					tab = this._tabs[i].tab;
					block = this._tabs[i].block;
				}
			}
			if(tab != undefined){
				this._selectTab(tab, block);
				if(this._sidebar !== undefined){
					this._sidebar.show();
				}
			}
		},


		currentTab: function(){
			if(this._lastTab !== undefined){
				return this._lastTab;
			}
			return false;
		}
});

