/*
	Copyright (c) 2006-2008, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.presentation.widget.Tray $Revision: 1.5.62.6 $

*/
dojo.provide("epages.cartridges.de_epages.presentation.widget.Tray");
dojo.require("epages.widget");
dojo.require("epages.io.json");
dojo.require('epages.html');
dojo.require('epages.format');
dojo.require('epages.widget.LocalizedWidget');

dojo.declare(
	"epages.cartridges.de_epages.presentation.widget.Tray",
	[epages.widget.LocalizedWidget],
	{
		objectId          : '',
		url               : '?',
		sidebarTabsId     : 'RightSidebarTabs',
		sidebarId         : 'RightSidebar',

		_groups           : undefined,
		_addObject        : undefined,
		_sidebarTabs      : undefined,
		_sidebar          : undefined,

		templatePath    : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/Tray.html'),
		imagePath       : epages.themeUrl('images'),
		translationName : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/translation'),

		postCreate  : function() {
			this.inherited("postCreate", arguments);
			dojo.subscribe('Sidebar/selectTab', this, '_onTabSelect');
			if($$(this.sidebarId)){
				this._sidebar = $$(this.sidebarId);
			}
			if($$(this.sidebarTabsId)){
				this._sidebarTabs = $$(this.sidebarTabsId);
				var tab = this._sidebarTabs.currentTab();
				//check if tab is selected and sidebar is visible before loading
				if(tab && this._sidebar.isVisible){
					this._onTabSelect(tab);
				}
			}else{
				this._loadTray();
				this._updateLayout();
			}
		},

		_onTabSelect:function(tab){
			if(tab && tab.menu == 'tray' && this._groups == undefined){
				this._loadTray();
				this._updateLayout();
			}
		},

		_loadTray: function(){
			var currentTray = this._sendRequest('JSONGetTray');
			if(currentTray){
				this._groups = currentTray.data;
				this._groups.sort(function(a, b) {
					if (a.Name < b.Name){
						return -1;
					}
					if (a.Name > b.Name){ 
						return  1;
					}
					return 0; 
				});
				this.containerNode.innerHTML = '';
				for( var i=0,iLength=this._groups.length ; i<iLength ; i++ ){
					var item = this.itemNode.cloneNode(true);
					item.style.display = '';
					var titleNode = epages.html.getElementsByClassName('ItemTitle',item)[0];
					titleNode.innerHTML = epages.format.slice(this._groups[i].Name,30)+" ("+this._groups[i].CountElements+")";
					titleNode.title = this._groups[i].Name;
					var deleteNode = epages.html.getElementsByClassName('ItemDeleteNode',item)[0];
					this.connect(deleteNode, 'onclick', 'deleteItem');
					this._groups[i].itemNode = item;
					this._groups[i].deleteNode = deleteNode;
					this.containerNode.appendChild(item);
				}
			}
		},

		_updateLayout: function(){
			if(this._groups.length == 0){
				this.emptyNode.style.display = '';
				this.deleteTrayNode.style.display = 'none';
				this.containerNode.style.display = 'none';
			}
			else{
				this.emptyNode.style.display = 'none';
				this.deleteTrayNode.style.display = '';
				this.containerNode.style.display = '';
			}
			if(this._addObject){
				this.showAddToTray(this._addObject);
			}
			else{
				this.existsNode.style.display = 'none';
				this.addNode.style.display = 'none';
			}
		},

		showAddToTray: function(addObject){
			this._addObject = addObject;
			if(this._groups){
				var exists = false;
				for( var i=0,iLength=this._groups.length ; i<iLength ; i++ ){
					for( var j=0,jLength=this._groups[i].Objects.length ; j<jLength ; j++ ){
						if(this._groups[i].Objects[j].ObjectID == addObject.objectId){
							exists = true;
						}
					}
				}
				if(exists){
					this.existsNode.style.display = '';
					this.existsNodeTitle.innerHTML = addObject.name;
					this.addNode.style.display = 'none';
				}
				else{
					this.addNode.style.display = '';
					this.addNodeTitle.innerHTML = addObject.name;
					this.existsNode.style.display = 'none';
				}
			}
		},

		deleteItem: function(evt){
			originalTarget = evt.originalTarget || evt.srcElement;
			var group = undefined;
			var index = undefined;
			for( var i=0,iLength=this._groups.length ; i<iLength ; i++ ){
				if(this._groups[i].deleteNode == originalTarget){
					group = this._groups[i];
					index = i;
				}
			}
			if(group){
				if(this._sendRequest('DeleteTrayGroup',{TrayGroupID:group.TrayGroupID})){
					this.containerNode.removeChild(group.itemNode);
					$A(this._groups).remove(index);
					this._updateLayout();
				}
			}
		},

		_deleteTray: function(){
			if(this._sendRequest('DeleteTray')){
				this._groups = [];
				this.containerNode.innerHTML = "";
				this._updateLayout();
			}
		},

		_sendRequest: function(changeAction,opt){
			var json = new epages.io.Json;
				var options = {
					'ViewAction':     'JSONViewResponse',
					'ChangeAction':   changeAction,
					'ObjectID':       this.objectId
				};
				if(opt){
					dojo.mixin(options,opt);
				}
				var result = json.loadSync(this.url,options);
				if(result.error){
					return false;
				}
				return result;
		},

		_addToTray: function(){
			if(this._addObject){
				if(this._sendRequest('AddListToTray',{ListObjectID:this._addObject.objectId})){
					var groupexists = false;
					for( var i=0,iLength=this._groups.length ; i<iLength ; i++ ){
						if(this._groups[i].Name == this._addObject.className){
							groupexists = true;
							this._groups[i].Objects.push({
								Alias: this._addObject.alias,
								Name: this._addObject.name,
								ObjectID: this._addObject.objectId
							});
							this._groups[i].CountElements++;
							var item = this._groups[i].itemNode;
							var titleNode = epages.html.getElementsByClassName('ItemTitle',item)[0];
							titleNode.innerHTML = this._groups[i].Name+" ("+this._groups[i].CountElements+")";
						}
					}
					if(!groupexists){
						this._loadTray();
					}
					this._updateLayout();
				}
			}
		}
});

