/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.
*/
dojo.provide("epages.cartridges.de_epages.presentation.widget.Undobuttons");
dojo.require("epages.widget.LocalizedWidget");
dojo.require("epages.html.undo");
dojo.require("epages.widget");

dojo.declare(
	"epages.cartridges.de_epages.presentation.widget.Undobuttons",
	[	epages.widget.LocalizedWidget ],
	{
		undoObject      : undefined,
		updateObject    : undefined,
		templatePath    : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/Undobuttons.html'),
		imagePath       : epages.themeUrl('images'),
		translationName : dojo.moduleUrl('epages.cartridges.de_epages.presentation.widget', 'templates/translation'),
		containerId     : "",
		validate        : null,
		siteId          : epages.vars.SiteID || '',
		url             : '?',
		isWizard        : false,

		postMixInProperties: function() {
			if ($$(this.id)) {
				// a widget with this id already exists
				var newId = '';
				var counter = 0;
				do {
					newId = this.id + '_' + counter;
				} while ($$(newId));
				this.id = newId;
			}
		},

		getUndoObject: function () {
			return this.undoObject;
		},

		postCreate: function() {
			if(this.containerId != "" && $(this.containerId) ){
				this.undoObject = new epages.html.UnDo($(this.containerId));
			}else{
				this.undoObject = new epages.html.UnDo();
			}
			dojo.addOnLoad(this, "init");
			this.inherited("postCreate", arguments);
		},

		init: function() {
			this.connect(this.undoObject, "validateInput", "toggleButtons");

			this.connect(this.undoObject, "undo", "toggleButtons");
			dojo.connect(this.undobutton, "onclick", this.undoObject, "undo");

			this.connect(this.undoObject, "redo", "toggleButtons");
			dojo.connect(this.redobutton, "onclick", this.undoObject, "redo");

			this.connect(this.savebutton, "onclick", "sendChanges");
			this.connect(this.savebutton, "onclick", "toggleButtons");

			this.connect(this.undoObject, "onChangeTyping", "toggleButtons");
		},

		toggleButtons: function(evt) {
			if (this.undoObject.historyStack.length != 0) {
				dojo.removeClass(this.undobutton, "DisabledButton");
			} else {
				dojo.addClass(this.undobutton, "DisabledButton");
			}
			if (this.undoObject.forwardStack.length != 0) {
				dojo.removeClass(this.redobutton, "DisabledButton");
			} else {
				dojo.addClass(this.redobutton, "DisabledButton");
			}
			if (this.undoObject.errorStack.length == 0 && (
						this.undoObject.isTyping ||
						this.undoObject.existsChanges()
					)
			) {
				dojo.removeClass(this.savebutton, "DisabledButton");
			} else {
				dojo.addClass(this.savebutton, "DisabledButton");
			}
		},

		sendChanges: function(messageOpt) {
			var message = this.translate("BusyWhileSavingData");
			var permanentMessage = false;
			var icon = epages.theme.url+'/images/ico_l_save.png';
			if(messageOpt) {
				if(messageOpt.text){ message = messageOpt.text; }
				if(messageOpt.icon){ icon = messageOpt.icon; }
				if(messageOpt.permanent){ permanentMessage = true; }
			}
			if (this.undoObject.errorStack.length == 0 && this.existsChanges()) {
				dojo.publish("uimessage/startBusy", ["Undobuttons", message, icon ]);
				if (this.updateObject === undefined) {
					dojo.require('epages.cartridges.de_epages.presentation.objectupdater');
					this.updateObject = new epages.cartridges.de_epages.presentation.ObjectUpdater({siteId:this.siteId});
				}
				this.updateObject.sendAsync(this.url, this.updateObject.getFormDataViaObjectForm(this.undoObject.getChanged()), function() {
					// dojo.publish("uimessage/stopBusy", ["Undobuttons"]);
					if(permanentMessage != true){
						window.setTimeout(function() {dojo.publish('uimessage/stopBusy', ["Undobuttons"]);} , 500);
					}
					dojo.publish('undobuttons/saveFinished', []);
				},this.isWizard);
				this.undoObject.checkTyping(false);
				this.undoObject.clear();
				this.toggleButtons();
			}// else  alert("errors: " + this.undoObject.errorStack.length);
		},

		existsChanges: function(){
			return this.undoObject.existsChanges();
		}
	}
);

epages.onUnloadFunctions.push(function() {
	var w = $$("undoWidget");
	if(w){
		return w.existsChanges();
	}
	return false;
});
