use strict;
use Test::Simple tests => 33;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( GetPKeyCartridgeByPackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadRootObject ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::Presentation::API::PageType qw (ExistsPageTypeByAlias LoadPageTypeByAlias);

sub Main {
    ConnectTestSite();
    ok(Init(),"INIT");

    # insert base page type
    my $ClassSystem = LoadClassByAlias( 'System' );
    my $Class =LoadClassByAlias( 'PageType' );
    my $oPageType1  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'TestBase',
        'PageTypeClass' => $ClassSystem,
    } );
    ok($oPageType1->id,"inserted PageTypeID Base ".$oPageType1->id);
    ok($oPageType1->get('PageTypeClass')->alias eq 'System', "set the correct class");

    # insert sub page type
    my $oPageType2  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'SubPageType',
        'PageTypeClass' => $ClassSystem,
        'BasePageType' => $oPageType1
    } );
    ok($oPageType2->id,"inserted SubPageTypeID ".$oPageType2->id);

    my $oPageType4  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'XSubPageType',
        'PageTypeClass' => $ClassSystem,
        'BasePageType' => $oPageType2
    } );
    ok($oPageType4->id,"inserted SubPageTypeID ".$oPageType4->id);

    my $oPageType5  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'XXSubPageType',
        'PageTypeClass' => $ClassSystem,
        'BasePageType' => $oPageType2
    } );
    ok($oPageType5->id,"inserted SubPageTypeID ".$oPageType5->id);

    # insert sub page type
    my $oPageType3  = LoadRootObject()->folder('PageTypes')->insertChild($Class,{
        'Alias'=>'SecSubPageType',
        'PageTypeClass' => $ClassSystem,
        'BasePageType' => $oPageType1
    } );
    ok($oPageType3->id,"inserted SecSubPageType ".$oPageType3->id);

    # exists name??
    ok( DE_EPAGES::Presentation::API::PageType::ExistsPageTypeByAlias($_),"$_ exists") foreach qw( TestBase SubPageType SecSubPageType XSubPageType);


    # test basePageType
    my $Base = $oPageType2->get('BasePageType');
    ok( $Base,"BaseID for ". $oPageType2->alias ." is ". $Base->id ." (".$Base->alias.")" );

    $oPageType4->set({'Alias'=>'XXXXXXXXXXSubPageType'});
    ok( $oPageType4->alias eq 'XXXXXXXXXXSubPageType', "PageType Alias updated" );


    #my $Base = $oPageType1->basePageType();  #undef!!
    my $CartridgeID = GetPKeyCartridgeByPackage("DE_EPAGES::Presentation");

    $oPageType5->insertTemplate({'Name'=>'ProviderDefault',
                                 'FileName'=>'System.Provider.default.TabPage.html',
                                 'CartridgeID'=>$CartridgeID
                                 });
    ok($oPageType5->existsTemplate('ProviderDefault'),"insert Template");

    $oPageType5->insertTemplate({'Name'=>'ProviderDefault1',
                                 'FileName'=>'System.Provider1.default.TabPage.html',
                                 'CartridgeID'=>$CartridgeID
                                 });
    ok($oPageType5->existsTemplate('ProviderDefault1'),"insert Template");

    $oPageType5->insertTemplate({'Name'=>'ProviderDefault2',
                                 'FileName'=>'System.Provider2.default.TabPage.html',
                                 'CartridgeID'=>$CartridgeID
                                 });
    ok($oPageType5->existsTemplate('ProviderDefault2'),"insert Template");


    $oPageType3->insertTemplate({'Name'=>'ProviderDefault3',
                                 'FileName'=>'System.Provider3.default.TabPage.html',
                                 'CartridgeID'=>$CartridgeID
                                 });
    ok($oPageType3->existsTemplate('ProviderDefault3'),"insert Template");

    # delet the template ProviderDefault2
    $oPageType5->deleteTemplate('ProviderDefault2');
    ok(!$oPageType5->existsTemplate('ProviderDefault2'),"delete Template ProviderDefault2");


    # attribut for ProviderDefault1
    my $hTemplate = $oPageType5->template('ProviderDefault1');
    ok( $hTemplate->{Name} eq 'ProviderDefault1', "template name" );
    ok( $hTemplate->{FileName} eq 'System.Provider1.default.TabPage.html', "template file name" );
    ok( $hTemplate->{PageTypeID} == $oPageType5->id, "template page type" );

    # all templates
    $oPageType5->templates();
    #logger($Tmpl1);


    # exists template for this page type ?
    ok( $oPageType5->existsTemplate($_),"$_ exists") foreach qw( ProviderDefault1 ProviderDefault);
    ok( !$oPageType5->existsTemplate($_),"$_ does not exists") foreach qw( ProviderDefault3 ProviderDefault2);

    $oPageType5->updateTemplate('ProviderDefault1', undef, {
        'FileName' => 'MyCustomerTabPage.html'
    });
    ok( $oPageType5->template('ProviderDefault1')->{FileName} eq 'MyCustomerTabPage.html', "updateTemplate" );

    # override the template for a sub class
    my $TestRoot = LoadObjectByPath('/Test');
    my $SubClass = $ClassSystem->insertSubClass({
        Alias => 'TestSubClass',
        Parent => $TestRoot,
    });
    my $SubClassObject = $SubClass->insertObject({
        Alias => 'SubClassObject',
        Parent => $TestRoot,
    });

    $oPageType5->setTemplateFile( {
        Name => 'ProviderDefault1',
        Object => $SubClass,
        FileName => 'ProviderDefault1-SubClass.html',
        CartridgeID => GetPKeyCartridgeByPackage('DE_EPAGES::Presentation'),
    } );
    ok( $oPageType5->template('ProviderDefault1', $SubClass)->{FileName} eq 'ProviderDefault1-SubClass.html', "template changed for class" );
    ok( $oPageType5->template('ProviderDefault1', $SubClassObject)->{FileName} eq 'ProviderDefault1-SubClass.html', "template changed for objects of class" );

    # override the template for an object
    $oPageType5->setTemplateFile( {
        Name => 'ProviderDefault1',
        Object => $SubClassObject,
        FileName => 'ProviderDefault1-SubClassObject.html',
        CartridgeID => GetPKeyCartridgeByPackage('DE_EPAGES::Presentation'),
    } );
    ok( $oPageType5->template('ProviderDefault1', $SubClass)->{FileName} eq 'ProviderDefault1-SubClass.html', "template not changed for class" );
    ok( $oPageType5->template('ProviderDefault1', $SubClassObject)->{FileName} eq 'ProviderDefault1-SubClassObject.html', "template changed for individual object" );

    # restore class default
    $oPageType5->setTemplateFile( {
        Name => 'ProviderDefault1',
        Object => $SubClassObject,
        FileName => undef,
        CartridgeID => GetPKeyCartridgeByPackage('DE_EPAGES::Presentation'),
    } );
    ok( $oPageType5->template('ProviderDefault1', $SubClassObject)->{FileName} eq 'ProviderDefault1-SubClass.html', "class template restored for individual object" );

    # restore global default
    $oPageType5->setTemplateFile( {
        Name => 'ProviderDefault1',
        Object => $SubClass,
        FileName => undef,
        CartridgeID => GetPKeyCartridgeByPackage('DE_EPAGES::Presentation'),
    } );
    ok( $oPageType5->template('ProviderDefault1', $SubClassObject)->{FileName} eq 'MyCustomerTabPage.html', "global template restored" );

    ok(Cleanup(),"CLEANUP");
    DisconnectSite();
}


sub logger {
    my ($elem) = @_;
    use Data::Dumper qw (Dumper);
    print("ELEM:\n".Dumper( $elem )."\n");

    return 1;
}


sub Cleanup {

    LoadPageTypeByAlias('TestBase')->delete if ExistsPageTypeByAlias('TestBase');
    LoadPageTypeByAlias('SubPageType')->delete if ExistsPageTypeByAlias('SubPageType');
    LoadPageTypeByAlias('XSubPageType')->delete if ExistsPageTypeByAlias('XSubPageType');
    LoadPageTypeByAlias('XXSubPageType')->delete if ExistsPageTypeByAlias('XXSubPageType');
    LoadPageTypeByAlias('SecSubPageType')->delete if ExistsPageTypeByAlias('SecSubPageType');
    LoadPageTypeByAlias('XXXXXXXXXXSubPageType')->delete if ExistsPageTypeByAlias('XXXXXXXXXXSubPageType');
    LoadObjectByPath('/Test')->delete if ExistsObjectByPath('/Test');

    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);

1;