use strict;
use Test::More;
use File::Find      qw ( find );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::File qw( GetFileContent );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::TLE::API::TLE qw ( GetByteCodeByText );
use DE_EPAGES::Core::API::File qw ( ExistsDirectory GetSubDirectories );

sub Main {

    ConnectTestSite();
    my $Directory = $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES';
    my @Files;
    my $aSubDirectories = GetSubDirectories($Directory);

    foreach my $TemplateDirectory (@$aSubDirectories){
        if (ExistsDirectory($TemplateDirectory . "/Templates")){
            find(sub {
                    return unless -f $File::Find::name;
                    return unless $File::Find::name =~ /\.html$/;
                    push @Files, $File::Find::name;
                },
                $TemplateDirectory . "/Templates"
            );
        }
    }

    plan tests => scalar @Files;
    foreach my $FileName (@Files) {
        eval {
            GetByteCodeByText(GetFileContent($FileName));
        };
        ok (!ExistsError(), $FileName);
        print STDERR GetError()->message, "\n" if ExistsError();
    }
    DisconnectSite();
}


RunScript(Sub=>\&Main, Debug=>1);

1;