use Test::Simple tests => 1;

use strict;
use DE_EPAGES::Core::API::File qw (DeleteFile);
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Presentation::API::XML::MonitoredExportDriver;
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Object::API::Language;
use DE_EPAGES::Core::API::Object::ProgressMonitor;

sub Main {
    
    ConnectTestSite();
    
    $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;
    
    my $System = LoadRootObject();
    my $Monitor = DE_EPAGES::Core::API::Object::ProgressMonitor->new(
        'ContentTemplate'   => "%2d of %d\n",
        'Total'             => scalar @{$System->children},
    );
    my $Exporter = DE_EPAGES::Presentation::API::XML::MonitoredExportDriver->new;
    $Exporter->exportXML(
        'test.xml', 
        { 
            Monitor => $Monitor,
        }, 
        $System, 
        $System->children
    );
    
    ok (scalar @{$System->children} == $Monitor->get('Current'), 'all exported');
    
    DisconnectSite();
    DeleteFile('test.xml');
}

RunScript(Sub=>\&Main, Debug=>1);
