use strict;
use Test::Simple tests => 12;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::WebService::API::Client;

sub Main {
    my $Proxy    = 'http://localhost:80/epages/Site.soap';
    my $User     = '/Providers/Distributor/Users/admin';
    my $Password = 'admin';
    
    # create a web service client
    my $ShopCountService = DE_EPAGES::WebService::API::Client
        ->uri('urn://epages.de/WebService/ShopCountService/2006/08')
        ->proxy($Proxy, timeout => 900 );
    $ShopCountService->userinfo( "$User:$Password" );

    ok( 1, "ShopCountService" );

    # count all shops
    my $TotalCount = $ShopCountService->getCount({})->result;
    ok( $TotalCount > 0, "total shop count" );

    # count only real live shops
    my $LiveCount = $ShopCountService->getCount({
        IsPreCreated => 0,
        IsClosed => 0,
        IsTrialShop => 0,
        IsInternalTestShop => 0,
        IsDeleted => 0,
    })->result;

    ok( $LiveCount > 0, "live shop count" );
    ok( $LiveCount <= $TotalCount, "live <= total" );

    my $ahCounters = $ShopCountService->getAllCounts({
        IsPreCreated => 0,
        IsClosed => 0,
        IsTrialShop => 0,
        IsInternalTestShop => 0,
        IsDeleted => 0,
    })->result;
    
    ok( scalar @$ahCounters > 0, "getAllCounts list is not empty" ); 

    my $hCounter = $ahCounters->[0];
    ok( defined $hCounter->{Database}, "getAllCounts[0].Database is defined" );
    ok( defined $hCounter->{ShopType}, "getAllCounts[0].ShopType is defined" );
    ok( defined $hCounter->{Trial_Closed}, "getAllCounts[0].Trial_Closed is defined" );
    ok( defined $hCounter->{Trial_NotClosed}, "getAllCounts[0].Trial_NotClosed is defined" );
    ok( defined $hCounter->{NotTrial_Closed}, "getAllCounts[0].NotTrial_Closed is defined" );
    ok( defined $hCounter->{NotTrial_NotClosed}, "getAllCounts[0].NotTrial_NotClosed is defined" );
    ok( defined $hCounter->{PreCreated}, "getAllCounts[0].PreCreated is defined" );
}

RunScript( Sub => \&Main );
