/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.
*/
dojo.provide("epages.widget.Saveinput");
dojo.require("epages.widget.Validateinput");
dojo.require('epages.widget.Button');

dojo.declare("epages.widget.Saveinput",
	[epages.widget.Validateinput],
	{
		url : "?",
		viewAction: "JSONViewResponse",
		changeAction: "Save",
		objectId : '',
		attribute: '',
		paramString: '', // eg. "{ StyleName : 'MyDesign' }"
		buttonLabel : "",
		buttonNode : undefined,
		buttonWidget : undefined,
		previewNode : undefined,

		_request : undefined,
		_lastErrorValue: undefined,

		postMixInProperties: function() {
			this.inherited("postMixInProperties", arguments);
			this._request = dojo.fromJson(this.paramString);
			this._request['ViewAction'] = this.viewAction;
			this._request['ChangeAction'] = this.changeAction;
			this._request['ObjectID'] = this.objectId;
		},

		postCreate: function() {
			this.inherited("postCreate", arguments);

			this.previewNode = document.createElement("span");
			this.previewNode.className = "SaveInputPreview ClickIcon";
			this.setPreviewContent();

			var bNode = document.createElement("input");
			bNode.type = "button";
			bNode.value= (this.buttonLabel != "") ? buttonLabel : this.dictionary.get("Save");
			dojo.addClass(this.domNode,"Saveinput");

			var nextSib = this.domNode.nextSibling;
			var parentNode = this.domNode.parentNode;

			if(nextSib) {
				parentNode.insertBefore(bNode,nextSib);
				parentNode.insertBefore(this.previewNode,nextSib);
			} else {
				parentNode.appendChild(bNode);
				parentNode.appendChild(this.previewNode);
			}
			this.buttonNode = bNode;

			var bWidget= new epages.widget.Button({},bNode);
			this.buttonWidget = bWidget;
			this.buttonWidget.hide();
			dojo.addClass(bWidget.domNode,"SaveinputButton");

			this.hideInput();

			this.connect(this.previewNode, "onclick", "showInput");
			this.connect(this.domNode, "onkeyup", "triggerButton");
			this.connect(bWidget.domNode, "onclick", "doSave");
		},

		setPreviewContent : function() {
			this.previewNode.innerHTML = this.domNode.value+'<i class="Sprite ico_s_edit"></i>';
		},

		showInput: function(evt) {
			this.domNode.style.display="";
			this.previewNode.style.display="none";
		},

		hideInput: function(evt) {
			this.domNode.style.display="none";
			this.previewNode.style.display="";
		},

		triggerButton: function(evt) {
			this.buttonWidget.show();
			this.buttonWidget.enable();
			if(this.getValue() != this._lastErrorValue){
				this._removeError();
			}else{
				this._setError();
			}
			if(evt.keyCode === dojo.keys.ENTER){
				this.doSave();
			}
		},

		getValue: function() {
			return this.domNode.value;
		},

		doSave: function() {
			var v=this.getValue();
			this._request[this.attribute] = v;
			if(v) {
				this.buttonWidget.disable();
				var request = new epages.io.Json();

				request.loadAsync(this.url,
					epages.lang.hitch(this, function (result) { this.requestOk(result);}),
					this._request,
					epages.lang.hitch(this, function (result, data) { this.requestFailed(result, data);})
				);
			}
		},

		requestOk: function(result) {
			var v = this.getValue();
			if(result.error) {
				var errors = result.error.data.Errors;
				this._setError();
				this.domNode.focus();
				this._lastErrorValue=v;
				dojo.publish("uimessage/showErrorBubble",[
					this.dictionary.get('Error'),
					this.domNode,
					errors ,
					 (errors[0].Reason=="OBJECT_EXISTS") ? "Warning" : undefined]
				);
			} else {
				this.setStartUpValue(v);
				this._lastErrorValue=undefined;
				this.buttonWidget.hide();
				this._removeError();
				this.setPreviewContent();
				this.hideInput();
			}
		},

		requestFailed: function(result) {
			var errors = result.data.Errors;
			if(errors) {
				dojo.publish("uimessage/showErrors", [
					this.dictionary.get('Error'),
					this.dictionary.get('Save'),
					errors]
				);
			} else {
				var err= result.error[0];
				var errors = [{
						Message: err.message,
						Reason: err.error
				}];

				dojo.publish("uimessage/showErrors", [
					this.dictionary.get('Error'),
					this.dictionary.get('Save'),
					errors]
				);
			}
		}

});
